@@ -12,18 +12,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from typing import Any, Dict, List, Optional
 
 from omegaconf import MISSING
 
 import nemo.core.classes.dataset
-from nemo.collections.asr.metrics.wer import CTCDecodingConfig
 from nemo.collections.asr.modules.audio_preprocessing import (
     AudioToMelSpectrogramPreprocessorConfig,
     SpectrogramAugmentationConfig,
 )
 from nemo.collections.asr.modules.conv_asr import ConvASRDecoderConfig, ConvASREncoderConfig
+from nemo.collections.asr.parts.submodules.ctc_decoding import CTCDecodingConfig
 from nemo.core.config import modelPT as model_cfg
 
 
@@ -41,6 +41,17 @@ class ASRDatasetConfig(nemo.core.classes.dataset.DatasetConfig):
     shard_manifests: bool = False
     shuffle_n: int = 0
 
+    # lhotse support
+    use_lhotse: bool = False
+    tarred_random_access: bool = False
+    use_bucketing: bool = False
+    batch_duration: Optional[int] = None
+    quadratic_duration: Optional[int] = None
+    bucket_batch_size: Optional[int] = None
+    bucket_duration_bins: Optional[list] = None
+    num_buckets: Optional[int] = 0
+    pin_memory: bool = False
+
     # Optional
     int_values: Optional[int] = None
     augmentor: Optional[Dict[str, Any]] = None
@@ -63,6 +74,9 @@ class ASRDatasetConfig(nemo.core.classes.dataset.DatasetConfig):
     bucketing_batch_size: Optional[Any] = None
     bucketing_weights: Optional[List[int]] = None
 
+    # Optional callable function to parse manifest file
+    manifest_parse_func: Optional[Any] = (None,)
+
 
 @dataclass
 class EncDecCTCConfig(model_cfg.ModelConfig):
@@ -74,37 +88,53 @@ class EncDecCTCConfig(model_cfg.ModelConfig):
     labels: List[str] = MISSING
 
     # Dataset configs
-    train_ds: ASRDatasetConfig = ASRDatasetConfig(manifest_filepath=None, shuffle=True)
-    validation_ds: ASRDatasetConfig = ASRDatasetConfig(manifest_filepath=None, shuffle=False)
-    test_ds: ASRDatasetConfig = ASRDatasetConfig(manifest_filepath=None, shuffle=False)
+    train_ds: ASRDatasetConfig = field(default_factory=lambda: ASRDatasetConfig(manifest_filepath=None, shuffle=True))
+    validation_ds: ASRDatasetConfig = field(
+        default_factory=lambda: ASRDatasetConfig(manifest_filepath=None, shuffle=False)
+    )
+    test_ds: ASRDatasetConfig = field(default_factory=lambda: ASRDatasetConfig(manifest_filepath=None, shuffle=False))
 
     # Optimizer / Scheduler config
-    optim: Optional[model_cfg.OptimConfig] = model_cfg.OptimConfig(sched=model_cfg.SchedConfig())
+    optim: Optional[model_cfg.OptimConfig] = field(
+        default_factory=lambda: model_cfg.OptimConfig(sched=model_cfg.SchedConfig())
+    )
 
     # Model component configs
-    preprocessor: AudioToMelSpectrogramPreprocessorConfig = AudioToMelSpectrogramPreprocessorConfig()
-    spec_augment: Optional[SpectrogramAugmentationConfig] = SpectrogramAugmentationConfig()
-    encoder: ConvASREncoderConfig = ConvASREncoderConfig()
-    decoder: ConvASRDecoderConfig = ConvASRDecoderConfig()
-    decoding: CTCDecodingConfig = CTCDecodingConfig()
+    preprocessor: AudioToMelSpectrogramPreprocessorConfig = field(
+        default_factory=lambda: AudioToMelSpectrogramPreprocessorConfig()
+    )
+    spec_augment: Optional[SpectrogramAugmentationConfig] = field(
+        default_factory=lambda: SpectrogramAugmentationConfig()
+    )
+    encoder: ConvASREncoderConfig = field(default_factory=lambda: ConvASREncoderConfig())
+    decoder: ConvASRDecoderConfig = field(default_factory=lambda: ConvASRDecoderConfig())
+    decoding: CTCDecodingConfig = field(default_factory=lambda: CTCDecodingConfig())
 
 
 @dataclass
 class EncDecCTCModelConfig(model_cfg.NemoConfig):
-    model: EncDecCTCConfig = EncDecCTCConfig()
+    model: EncDecCTCConfig = field(default_factory=lambda: EncDecCTCConfig())
 
 
 @dataclass
 class CacheAwareStreamingConfig:
-    chunk_size: int = 0  # the size of each chunk at each step, it can be a list of two integers to specify different chunk sizes for the first step and others
-    shift_size: int = 0  # the size of the shift in each step, it can be a list of two integers to specify different shift sizes for the first step and others
+    chunk_size: int = (
+        0  # the size of each chunk at each step, it can be a list of two integers to specify different chunk sizes for the first step and others
+    )
+    shift_size: int = (
+        0  # the size of the shift in each step, it can be a list of two integers to specify different shift sizes for the first step and others
+    )
 
     cache_drop_size: int = 0  # the number of steps to drop from the cache
     last_channel_cache_size: int = 0  # the size of the needed cache for last channel layers
 
-    valid_out_len: int = 0  # the number of the steps in the final output which are valid (have the same value as in the offline mode)
+    valid_out_len: int = (
+        0  # the number of the steps in the final output which are valid (have the same value as in the offline mode)
+    )
 
-    pre_encode_cache_size: int = 0  # the size of the needed cache for the pre-encoding part of the model to avoid caching inside the pre-encoding layers
+    pre_encode_cache_size: int = (
+        0  # the size of the needed cache for the pre-encoding part of the model to avoid caching inside the pre-encoding layers
+    )
     drop_extra_pre_encoded: int = 0  # the number of steps to get dropped after the pre-encoding layer
 
     last_channel_num: int = 0  # number of the last channel layers (like MHA layers) which need caching in the model
