@@ -12,15 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from nemo.collections.asr.modules.audio_modules import MaskBasedBeamformer, MaskEstimatorRNN, MaskReferenceChannel
 from nemo.collections.asr.modules.audio_preprocessing import (
     AudioToMelSpectrogramPreprocessor,
     AudioToMFCCPreprocessor,
-    AudioToSpectrogram,
     CropOrPadSpectrogramAugmentation,
     MaskedPatchAugmentation,
     SpectrogramAugmentation,
-    SpectrogramToAudio,
 )
 from nemo.collections.asr.modules.beam_search_decoder import BeamSearchDecoderWithLM
 from nemo.collections.asr.modules.conformer_encoder import ConformerEncoder, ConformerEncoderAdapter
@@ -47,3 +44,11 @@ from nemo.collections.asr.modules.rnnt import (
     StatelessTransducerDecoder,
 )
 from nemo.collections.asr.modules.squeezeformer_encoder import SqueezeformerEncoder, SqueezeformerEncoderAdapter
+from nemo.collections.asr.modules.ssl_modules import (
+    ConformerMultiLayerFeatureExtractor,
+    ConformerMultiLayerFeaturePreprocessor,
+    ConvFeatureMaksingWrapper,
+    MultiSoftmaxDecoder,
+    RandomBlockMasking,
+    RandomProjectionVectorQuantizer,
+)
