@@ -12,24 +12,28 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import math
 import os
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from typing import List, Optional, Tuple, Union
 
 import torch
 
+from nemo.collections.asr.parts.k2.classes import GraphIntersectDenseConfig
+from nemo.collections.asr.parts.submodules.ngram_lm import DEFAULT_TOKEN_OFFSET
+from nemo.collections.asr.parts.submodules.wfst_decoder import RivaDecoderConfig, WfstNbestHypothesis
 from nemo.collections.asr.parts.utils import rnnt_utils
 from nemo.collections.common.tokenizers.tokenizer_spec import TokenizerSpec
 from nemo.core.classes import Typing, typecheck
 from nemo.core.neural_types import HypothesisType, LengthsType, LogprobsType, NeuralType
 from nemo.utils import logging
 
-DEFAULT_TOKEN_OFFSET = 100
-
 
 def pack_hypotheses(
-    hypotheses: List[rnnt_utils.NBestHypotheses], logitlen: torch.Tensor,
+    hypotheses: List[rnnt_utils.NBestHypotheses],
+    logitlen: torch.Tensor,
 ) -> List[rnnt_utils.NBestHypotheses]:
 
     if logitlen is not None:
@@ -51,6 +55,39 @@ def pack_hypotheses(
     return hypotheses
 
 
+def pack_wfst_hypotheses(
+    hypotheses: List['WfstNbestHypothesis'],
+    logits: torch.Tensor,
+    logitlen: torch.Tensor,
+) -> List[rnnt_utils.NBestHypotheses]:
+
+    logitlen_cpu = logitlen.to('cpu')
+
+    new_hypotheses = []
+    for idx, nbest_hyp in enumerate(hypotheses):  # type: WfstNbestHypothesis
+        new_hyp = []
+        y_sequence = logits[idx, : logitlen[idx]].to('cpu')
+        length = logitlen_cpu[idx]
+        for candidate_idx, cand in enumerate(nbest_hyp):
+            cand_hyp = rnnt_utils.Hypothesis(
+                y_sequence=[],
+                score=cand.score,
+                text=" ".join(cand.words),
+                timestamp=list(cand.timesteps),
+                alignments=list(cand.alignment),
+            )
+            cand_hyp.y_sequence = y_sequence
+
+            if logitlen is not None:
+                cand_hyp.length = length
+
+            new_hyp.append(cand_hyp)
+
+        new_hypotheses.append(rnnt_utils.NBestHypotheses(new_hyp))
+
+    return new_hypotheses
+
+
 def _states_to_device(dec_state, device='cpu'):
     if torch.is_tensor(dec_state):
         dec_state = dec_state.to(device)
@@ -74,8 +111,7 @@ class AbstractBeamCTCInfer(Typing):
 
     @property
     def input_types(self):
-        """Returns definitions of module input ports.
-        """
+        """Returns definitions of module input ports."""
         return {
             "decoder_output": NeuralType(('B', 'T', 'D'), LogprobsType()),
             "decoder_lengths": NeuralType(tuple('B'), LengthsType()),
@@ -83,8 +119,7 @@ class AbstractBeamCTCInfer(Typing):
 
     @property
     def output_types(self):
-        """Returns definitions of module output ports.
-        """
+        """Returns definitions of module output ports."""
         return {"predictions": [NeuralType(elements_type=HypothesisType())]}
 
     def __init__(self, blank_id: int, beam_size: int):
@@ -147,7 +182,9 @@ class AbstractBeamCTCInfer(Typing):
 
     @typecheck()
     def forward(
-        self, decoder_output: torch.Tensor, decoder_lengths: torch.Tensor,
+        self,
+        decoder_output: torch.Tensor,
+        decoder_lengths: torch.Tensor,
     ) -> Tuple[List[Union[rnnt_utils.Hypothesis, rnnt_utils.NBestHypotheses]]]:
         """Returns a list of hypotheses given an input batch of the encoder hidden embedding.
         Output token is generated auto-repressively.
@@ -204,7 +241,7 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
         self.compute_timestamps = compute_timestamps
 
         if self.compute_timestamps:
-            raise ValueError(f"Currently this flag is not supported for beam search algorithms.")
+            raise ValueError("Currently this flag is not supported for beam search algorithms.")
 
         self.vocab = None  # This must be set by specific method by user before calling forward() !
 
@@ -246,7 +283,9 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
 
     @typecheck()
     def forward(
-        self, decoder_output: torch.Tensor, decoder_lengths: torch.Tensor,
+        self,
+        decoder_output: torch.Tensor,
+        decoder_lengths: torch.Tensor,
     ) -> Tuple[List[Union[rnnt_utils.Hypothesis, rnnt_utils.NBestHypotheses]]]:
         """Returns a list of hypotheses given an input batch of the encoder hidden embedding.
         Output token is generated auto-repressively.
@@ -349,7 +388,7 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
             hypotheses = []
             for candidate_idx, candidate in enumerate(beams):
                 hypothesis = rnnt_utils.Hypothesis(
-                    score=0.0, y_sequence=[], dec_state=None, timestep=[], last_token=None
+                    score=0.0, y_sequence=[], dec_state=None, timestamp=[], last_token=None
                 )
 
                 # For subword encoding, NeMo will double encode the subword (multiple tokens) into a
@@ -406,8 +445,8 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
             import pyctcdecode
         except (ImportError, ModuleNotFoundError):
             raise ImportError(
-                f"Could not load `pyctcdecode` library. Please install it from pip using :\n"
-                f"pip install --upgrade pyctcdecode"
+                "Could not load `pyctcdecode` library. Please install it from pip using :\n"
+                "pip install --upgrade pyctcdecode"
             )
 
         if self.pyctcdecode_beam_scorer is None:
@@ -439,7 +478,7 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
             for candidate_idx, candidate in enumerate(beams):
                 # Candidate = (text, last_lm_state, text_frames, logit_score, lm_score)
                 hypothesis = rnnt_utils.Hypothesis(
-                    score=0.0, y_sequence=[], dec_state=None, timestep=[], last_token=None
+                    score=0.0, y_sequence=[], dec_state=None, timestamp=[], last_token=None
                 )
 
                 # TODO: Requires token ids to be returned rather than text.
@@ -460,7 +499,7 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
                 hypothesis.score = candidate[4]  # score
 
                 # Inject word level timestamps
-                hypothesis.timestep = candidate[2]  # text_frames
+                hypothesis.timestamp = candidate[2]  # text_frames
 
                 if self.preserve_alignments:
                     hypothesis.alignments = torch.from_numpy(x[beams_idx][: out_len[beams_idx]])
@@ -497,7 +536,7 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
             if self.kenlm_path is None or not os.path.exists(self.kenlm_path):
                 raise FileNotFoundError(
                     f"KenLM binary file not found at : {self.kenlm_path}. "
-                    f"Please set a valid path in the decoding config."
+                    "Please set a valid path in the decoding config."
                 )
 
             # perform token offset for subword models
@@ -537,7 +576,7 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
             hypotheses = []
             for candidate_idx, candidate in enumerate(beams):
                 hypothesis = rnnt_utils.Hypothesis(
-                    score=0.0, y_sequence=[], dec_state=None, timestep=[], last_token=None
+                    score=0.0, y_sequence=[], dec_state=None, timestamp=[], last_token=None
                 )
 
                 # We preserve the token ids and the score for this hypothesis
@@ -568,6 +607,276 @@ class BeamCTCInfer(AbstractBeamCTCInfer):
             self.token_offset = DEFAULT_TOKEN_OFFSET
 
 
+class WfstCTCInfer(AbstractBeamCTCInfer):
+    """A WFST-based beam CTC decoder.
+
+    Provides a common abstraction for sample level and batch level beam decoding.
+
+    Args:
+        TBD
+
+    """
+
+    def __init__(
+        self,
+        blank_id: int,
+        beam_size: int,
+        search_type: str = "riva",  # 'riva', 'k2'
+        return_best_hypothesis: bool = True,
+        preserve_alignments: bool = False,
+        compute_timestamps: bool = False,
+        decoding_mode: str = 'nbest',  # 'nbest', 'mbr' ('mbr' works only for search_type == 'riva' and beam_size == 1)
+        open_vocabulary_decoding: bool = False,
+        beam_width: float = 10.0,
+        lm_weight: float = 1.0,
+        device: str = "cuda",
+        arpa_lm_path: str = None,
+        wfst_lm_path: str = None,
+        riva_decoding_cfg: Optional['RivaDecoderConfig'] = None,
+        k2_decoding_cfg: Optional['GraphIntersectDenseConfig'] = None,
+    ):
+        super().__init__(blank_id=blank_id, beam_size=beam_size)
+
+        self.search_type = search_type
+        self.return_best_hypothesis = return_best_hypothesis
+        self.preserve_alignments = preserve_alignments
+        self.compute_timestamps = compute_timestamps
+
+        self.decoding_algorithm = None
+        if search_type in ("default", "riva"):
+            self.decoding_algorithm = self._riva_decoding
+        elif search_type == "k2":
+            self.decoding_algorithm = self._k2_decoding
+
+        # Log the WFST search_type
+        logging.info(f"WFST beam search search_type: {search_type}")
+        self.search_type = search_type
+
+        if beam_size > 1 and decoding_mode != 'nbest':
+            logging.warning(
+                f"`beam_size` > 1 is supported only for `decoding_mode` == `nbest`\n"
+                f"(provided: `{decoding_mode}`).\n"
+                f"`beam_size` rewritten as 1"
+            )
+            self.beam_size = 1
+        self.decoding_mode = decoding_mode
+
+        self.open_vocabulary_decoding = open_vocabulary_decoding
+        self._tokenword_disambig_id = -1
+        self.beam_width = beam_width
+        self.lm_weight = lm_weight
+        self.device = device
+
+        # Default beam search args
+        self.arpa_lm_path = arpa_lm_path
+        self.wfst_lm_path = wfst_lm_path
+
+        self.riva_decoding_cfg = riva_decoding_cfg
+        self.k2_decoding_cfg = k2_decoding_cfg
+
+        # Default beam search scorer functions
+        self.riva_decoder = None
+        self.k2_decoder = None
+
+    @typecheck()
+    def forward(
+        self,
+        decoder_output: torch.Tensor,
+        decoder_lengths: torch.Tensor,
+    ) -> Tuple[List[Union[rnnt_utils.Hypothesis, rnnt_utils.NBestHypotheses]]]:
+        """Returns a list of hypotheses given an input batch of the encoder hidden embedding.
+        Output token is generated auto-repressively.
+
+        Args:
+            decoder_output: A tensor of size (batch, timesteps, features).
+            decoder_lengths: list of int representing the length of each sequence
+                output sequence.
+
+        Returns:
+            packed list containing batch number of sentences (Hypotheses).
+        """
+        if self.vocab is None:
+            raise RuntimeError("Please set the vocabulary with `set_vocabulary()` before calling this function.")
+
+        if self.decoding_type != 'subword':
+            raise ValueError(
+                f"`decoding_type` other than `subword` is not supported. Provided: `{self.decoding_type}`"
+            )
+        elif self.tokenizer is None:
+            raise ValueError("Tokenizer must be provided for subword decoding. Use set_tokenizer().")
+        if self.decoding_algorithm is None:
+            raise NotImplementedError(
+                f"The decoding search_type ({self.search_type}) supplied is not supported!\n"
+                f"Please use one of : (default, riva, k2)"
+            )
+
+        with torch.no_grad(), torch.inference_mode():
+            # Process each sequence independently
+            prediction_tensor = decoder_output
+
+            if prediction_tensor.ndim != 3:
+                raise ValueError(
+                    f"`decoder_output` must be a tensor of shape [B, T, V] (log probs, float). "
+                    f"Provided shape = {prediction_tensor.shape}"
+                )
+
+            hypotheses = self.decoding_algorithm(prediction_tensor, decoder_lengths)
+
+            # Pack results into Hypotheses
+            packed_result = pack_wfst_hypotheses(hypotheses, prediction_tensor, decoder_lengths)
+
+            # Pack the result
+            if self.return_best_hypothesis and isinstance(packed_result[0], rnnt_utils.NBestHypotheses):
+                packed_result = [res.n_best_hypotheses[0] for res in packed_result]  # type: Hypothesis
+
+        return (packed_result,)
+
+    def _prepare_decoding_lm_wfst(self) -> Union[str, 'kaldifst.StdFst', 'k2.Fsa']:  # noqa: F821
+        """TBD"""
+        arpa_lm_path_exists = self.arpa_lm_path is not None and os.path.exists(self.arpa_lm_path)
+        wfst_lm_path_exists = self.wfst_lm_path is not None and os.path.exists(self.wfst_lm_path)
+        lm_fst = None
+        if wfst_lm_path_exists:
+            if self.search_type == "riva" and not self.wfst_lm_path.endswith(".fst"):
+                raise ValueError(
+                    f"Search type `riva` expects WFSTs in the `.fst` format. Provided: `{self.wfst_lm_path}`"
+                )
+            if self.search_type == "k2" and not self.wfst_lm_path.endswith(".pt"):
+                raise ValueError(
+                    f"Search type `k2` expects WFSTs in the `.pt` format. Provided: `{self.wfst_lm_path}`"
+                )
+            if arpa_lm_path_exists:
+                logging.warning(
+                    "Both `arpa_lm_path` and `wfst_lm_path` are provided and not empty. The latter will be used."
+                )
+            lm_fst = self.wfst_lm_path
+        elif not arpa_lm_path_exists:
+            raise FileNotFoundError(
+                f"Arpa LM file not found at `{self.arpa_lm_path}` and WFST LM is not found at `{self.wfst_lm_path}`.\n"
+                f"Please set a valid path in the decoding config for at least one of those."
+            )
+        else:
+            logging.warning(
+                f"Since WFST LM is not found at `{self.wfst_lm_path}`, "
+                f"it will be made from the Arpa LM at `{self.arpa_lm_path}`.\n"
+                f"This procedure will take some time."
+            )
+            if self.wfst_lm_path is not None:
+                logging.info(f"WFST LM will be buffered at `{self.wfst_lm_path}`.")
+                write_tlg_path = self.wfst_lm_path
+            else:
+                logging.warning("Consider providing a write-permitted `wfst_lm_path` for WFST LM buffering.")
+                write_tlg_path = None
+            ctc_topology = "default"  # there is no way to indicate the need of other topologies
+            target = "kaldi" if self.search_type == "riva" else "k2"
+
+            from nemo.collections.asr.parts.utils.wfst_utils import mkgraph_ctc_ov
+
+            lm_fst, tokenword_disambig_id = mkgraph_ctc_ov(
+                tokenizer=self.tokenizer,
+                lm_path=self.arpa_lm_path,
+                topology_name=ctc_topology,
+                write_tlg_path=write_tlg_path,
+                open_vocabulary=self.open_vocabulary_decoding,
+                target=target,
+            )
+            self._tokenword_disambig_id = tokenword_disambig_id
+
+        return lm_fst
+
+    @torch.no_grad()
+    def _riva_decoding(self, x: torch.Tensor, out_len: torch.Tensor) -> List['WfstNbestHypothesis']:
+        """
+        Riva Asrlib WFST decoder Algorithm.
+
+        Args:
+            x: Tensor of shape [B, T, V+1], where B is the batch size, T is the maximum sequence length,
+                and V is the vocabulary size. The tensor contains log-probabilities.
+            out_len: Tensor of shape [B], contains lengths of each sequence in the batch.
+
+        Returns:
+            A list of WfstNbestHypothesis objects, one for each sequence in the batch.
+        """
+        if self.riva_decoder is None:
+            lm_fst = self._prepare_decoding_lm_wfst()
+            if self.open_vocabulary_decoding and self._tokenword_disambig_id == -1:
+                # trying to extract tokenword_disambig_id from the lm_fst
+                if isinstance(lm_fst, str):
+                    # use importer instead of direct import to possibly get an installation message
+                    from nemo.collections.asr.parts.utils.wfst_utils import kaldifst_importer
+
+                    kaldifst = kaldifst_importer()
+                    lm_fst = kaldifst.StdVectorFst.read(self.wfst_lm_path)
+                tokenword_disambig_id = lm_fst.output_symbols.find("#1")
+                if tokenword_disambig_id == -1:
+                    raise ValueError(
+                        "Cannot determine `tokenword_disambig_id` "
+                        "which is required if `open_vocabulary_decoding` == True"
+                    )
+                self._tokenword_disambig_id = tokenword_disambig_id
+            if not self.device.startswith("cuda"):
+                raise ValueError(f"Riva decoder does not support non-cuda device. Provided: `{self.device}`")
+
+            from nemo.collections.asr.parts.submodules.wfst_decoder import RivaGpuWfstDecoder
+
+            self.riva_decoder = RivaGpuWfstDecoder(
+                lm_fst=lm_fst,
+                decoding_mode=self.decoding_mode,
+                beam_size=self.beam_width,
+                config=self.riva_decoding_cfg,
+                tokenword_disambig_id=self._tokenword_disambig_id,
+                lm_weight=self.lm_weight,
+                nbest_size=self.beam_size,
+            )
+
+        return self.riva_decoder.decode(x.to(device=self.device), out_len.to(device=self.device))
+
+    @torch.no_grad()
+    def _k2_decoding(self, x: torch.Tensor, out_len: torch.Tensor) -> List['WfstNbestHypothesis']:
+        """
+        K2 WFST decoder Algorithm.
+
+        Args:
+            x: Tensor of shape [B, T, V+1], where B is the batch size, T is the maximum sequence length,
+                and V is the vocabulary size. The tensor contains log-probabilities.
+            out_len: Tensor of shape [B], contains lengths of each sequence in the batch.
+
+        Returns:
+            A list of WfstNbestHypothesis objects, one for each sequence in the batch.
+        """
+        if self.k2_decoder is None:
+            lm_fst = self._prepare_decoding_lm_wfst()
+            if self.open_vocabulary_decoding and self._tokenword_disambig_id == -1:
+                if isinstance(lm_fst, str):
+                    from nemo.collections.asr.parts.k2.utils import load_graph
+
+                    with torch.inference_mode(False):
+                        lm_fst = load_graph(lm_fst)
+                try:
+                    tokenword_disambig_id = lm_fst.aux_labels_sym.get("#1")
+                    self._tokenword_disambig_id = tokenword_disambig_id
+                except KeyError:
+                    raise ValueError(
+                        "Cannot determine `tokenword_disambig_id` "
+                        "which is required if `open_vocabulary_decoding` == True"
+                    )
+
+            from nemo.collections.asr.parts.k2.graph_decoders import K2WfstDecoder
+
+            self.k2_decoder = K2WfstDecoder(
+                lm_fst=lm_fst,
+                decoding_mode=self.decoding_mode,
+                beam_size=self.beam_width,
+                config=self.k2_decoding_cfg,
+                tokenword_disambig_id=self._tokenword_disambig_id,
+                lm_weight=self.lm_weight,
+                nbest_size=self.beam_size,
+                device=self.device,
+            )
+
+        return self.k2_decoder.decode(x.to(device=self.device), out_len.to(device=self.device))
+
+
 @dataclass
 class PyCTCDecodeConfig:
     # These arguments cannot be imported from pyctcdecode (optional dependency)
@@ -602,5 +911,23 @@ class BeamCTCInferConfig:
     beam_beta: float = 0.0
     kenlm_path: Optional[str] = None
 
-    flashlight_cfg: Optional[FlashlightConfig] = FlashlightConfig()
-    pyctcdecode_cfg: Optional[PyCTCDecodeConfig] = PyCTCDecodeConfig()
+    flashlight_cfg: Optional[FlashlightConfig] = field(default_factory=lambda: FlashlightConfig())
+    pyctcdecode_cfg: Optional[PyCTCDecodeConfig] = field(default_factory=lambda: PyCTCDecodeConfig())
+
+
+@dataclass
+class WfstCTCInferConfig:
+    beam_size: int
+    search_type: str = "riva"  # 'riva', 'k2'
+    return_best_hypothesis: bool = True
+    preserve_alignments: bool = False
+    compute_timestamps: bool = False
+    decoding_mode: str = 'nbest'  # 'nbest', 'mbr' ('mbr' works only for search_type == 'riva' and beam_size == 1)
+    open_vocabulary_decoding: bool = False
+    beam_width: float = 10.0
+    lm_weight: float = 1.0
+    device: str = "cuda"
+    arpa_lm_path: Optional[str] = None
+    wfst_lm_path: Optional[str] = None
+    riva_decoding_cfg: Optional['RivaDecoderConfig'] = field(default_factory=lambda: RivaDecoderConfig())
+    k2_decoding_cfg: Optional['GraphIntersectDenseConfig'] = field(default_factory=lambda: GraphIntersectDenseConfig())
