@@ -16,6 +16,8 @@ import json
 import os
 import re
 from dataclasses import dataclass
+from pathlib import Path
+from tempfile import NamedTemporaryFile
 from typing import List, Optional, Tuple, Union
 
 import torch
@@ -23,10 +25,11 @@ from omegaconf import DictConfig
 from tqdm.auto import tqdm
 
 import nemo.collections.asr as nemo_asr
-from nemo.collections.asr.models import ASRModel
-from nemo.collections.asr.models.ctc_models import EncDecCTCModel
-from nemo.collections.asr.parts.utils import rnnt_utils
-from nemo.collections.asr.parts.utils.streaming_utils import FrameBatchASR
+from nemo.collections.asr.metrics.wer import word_error_rate
+from nemo.collections.asr.models import ASRModel, EncDecMultiTaskModel
+from nemo.collections.asr.parts.utils import manifest_utils, rnnt_utils
+from nemo.collections.asr.parts.utils.streaming_utils import FrameBatchASR, FrameBatchMultiTaskAED
+from nemo.collections.common.metrics.punct_er import OccurancePunctuationErrorRate
 from nemo.collections.common.parts.preprocessing.manifest import get_full_path
 from nemo.utils import logging, model_utils
 
@@ -39,6 +42,7 @@ def get_buffered_pred_feat_rnnt(
     batch_size: int,
     manifest: str = None,
     filepaths: List[list] = None,
+    accelerator: Optional[str] = 'cpu',
 ) -> List[rnnt_utils.Hypothesis]:
     """
     Moved from examples/asr/asr_chunked_inference/rnnt/speech_to_text_buffered_infer_rnnt.py
@@ -56,15 +60,18 @@ def get_buffered_pred_feat_rnnt(
         filepaths = []
         with open(manifest, "r", encoding='utf_8') as mfst_f:
             print("Parsing manifest files...")
-            for l in mfst_f:
-                row = json.loads(l.strip())
+            for L in mfst_f:
+                L = L.strip()
+                if not L:
+                    continue
+                row = json.loads(L)
                 audio_file = get_full_path(audio_file=row['audio_filepath'], manifest_file=manifest)
                 filepaths.append(audio_file)
                 if 'text' in row:
                     refs.append(row['text'])
 
     with torch.inference_mode():
-        with torch.cuda.amp.autocast():
+        with torch.amp.autocast('cpu' if accelerator == 'cpu' else 'cuda'):
             batch = []
             asr.sample_offset = 0
             for idx in tqdm(range(len(filepaths)), desc='Sample:', total=len(filepaths)):
@@ -138,16 +145,19 @@ def get_buffered_pred_feat(
         raise ValueError("Either filepaths or manifest shoud not be None")
 
     if filepaths:
-        for l in tqdm(filepaths, desc="Sample:"):
+        for L in tqdm(filepaths, desc="Sample:"):
             asr.reset()
-            asr.read_audio_file(l, delay, model_stride_in_secs)
+            asr.read_audio_file(L, delay, model_stride_in_secs)
             hyp = asr.transcribe(tokens_per_chunk, delay)
             hyps.append(hyp)
     else:
         with open(manifest, "r", encoding='utf_8') as mfst_f:
-            for l in tqdm(mfst_f, desc="Sample:"):
+            for L in tqdm(mfst_f, desc="Sample:"):
                 asr.reset()
-                row = json.loads(l.strip())
+                L = L.strip()
+                if not L:
+                    continue
+                row = json.loads(L)
                 if 'text' in row:
                     refs.append(row['text'])
                 audio_file = get_full_path(audio_file=row['audio_filepath'], manifest_file=manifest)
@@ -170,8 +180,80 @@ def get_buffered_pred_feat(
     return wrapped_hyps
 
 
+def get_buffered_pred_feat_multitaskAED(
+    asr: FrameBatchMultiTaskAED,
+    preprocessor_cfg: DictConfig,
+    model_stride_in_secs: int,
+    device: Union[List[int], int],
+    manifest: str = None,
+    filepaths: List[list] = None,
+    delay: float = 0.0,
+    timestamps: bool = False,
+) -> List[rnnt_utils.Hypothesis]:
+    # Create a preprocessor to convert audio samples into raw features,
+    # Normalization will be done per buffer in frame_bufferer
+    # Do not normalize whatever the model's preprocessor setting is
+    preprocessor_cfg.normalize = "None"
+    preprocessor = EncDecMultiTaskModel.from_config_dict(preprocessor_cfg)
+    preprocessor.to(device)
+    hyps = []
+    refs = []
+
+    if filepaths and manifest:
+        raise ValueError("Please select either filepaths or manifest")
+    if filepaths is None and manifest is None:
+        raise ValueError("Either filepaths or manifest shoud not be None")
+
+    if filepaths:
+        logging.info(
+            "Deteced audio files as input, default to English ASR with Punctuation and Capitalization output. \
+                Please use manifest input for other options."
+        )
+        for audio_file in tqdm(filepaths, desc="Transcribing:", total=len(filepaths), ncols=80):
+            meta = {
+                'audio_filepath': audio_file,
+                'duration': 100000,
+                'source_lang': 'en',
+                'taskname': 'asr',
+                'target_lang': 'en',
+                'pnc': 'yes',
+                'answer': 'nothing',
+                'timestamp': 'yes' if timestamps else 'no',
+            }
+            asr.reset()
+            asr.read_audio_file(audio_file, delay, model_stride_in_secs, meta_data=meta)
+            hyp = asr.transcribe()
+            hyps.append(hyp)
+    else:
+        with open(manifest, "r", encoding='utf_8') as fin:
+            lines = list(fin.readlines())
+            for line in tqdm(lines, desc="Transcribing:", total=len(lines), ncols=80):
+                asr.reset()
+                line = line.strip()
+                if not line:
+                    continue
+                sample = json.loads(line)
+                if (
+                    timestamps
+                ):  # user convenience so that they don't need to make another manifest with timestamp field or modify the existing one
+                    sample['timestamp'] = 'yes'
+                if 'text' in sample:
+                    refs.append(sample['text'])
+                audio_file = get_full_path(audio_file=sample['audio_filepath'], manifest_file=manifest)
+                # do not support partial audio
+                asr.read_audio_file(audio_file, delay, model_stride_in_secs, meta_data=sample)
+                hyp = asr.transcribe()
+                hyps.append(hyp)
+
+    wrapped_hyps = wrap_transcription(hyps)
+    return wrapped_hyps
+
+
 def wrap_transcription(hyps: List[str]) -> List[rnnt_utils.Hypothesis]:
-    """ Wrap transcription to the expected format in func write_transcription """
+    """Wrap transcription to the expected format in func write_transcription"""
+    if isinstance(hyps[0], rnnt_utils.Hypothesis):
+        return hyps
+
     wrapped_hyps = []
     for hyp in hyps:
         hypothesis = rnnt_utils.Hypothesis(score=0.0, y_sequence=[], text=hyp)
@@ -180,7 +262,7 @@ def wrap_transcription(hyps: List[str]) -> List[rnnt_utils.Hypothesis]:
 
 
 def setup_model(cfg: DictConfig, map_location: torch.device) -> Tuple[ASRModel, str]:
-    """ Setup model from cfg and return model and model name for next step """
+    """Setup model from cfg and return model and model name for next step"""
     if cfg.model_path is not None and cfg.model_path != "None":
         # restore model from .nemo file path
         model_cfg = ASRModel.restore_from(restore_path=cfg.model_path, return_config=True)
@@ -188,57 +270,113 @@ def setup_model(cfg: DictConfig, map_location: torch.device) -> Tuple[ASRModel,
         imported_class = model_utils.import_class_by_path(classpath)  # type: ASRModel
         logging.info(f"Restoring model : {imported_class.__name__}")
         asr_model = imported_class.restore_from(
-            restore_path=cfg.model_path, map_location=map_location,
+            restore_path=cfg.model_path,
+            map_location=map_location,
         )  # type: ASRModel
-        if hasattr(cfg, "model_change"):
-            asr_model.change_attention_model(
-                self_attention_model=cfg.model_change.conformer.get("self_attention_model", None),
-                att_context_size=cfg.model_change.conformer.get("att_context_size", None),
-            )
         model_name = os.path.splitext(os.path.basename(cfg.model_path))[0]
     else:
         # restore model by name
         asr_model = ASRModel.from_pretrained(
-            model_name=cfg.pretrained_name, map_location=map_location,
+            model_name=cfg.pretrained_name,
+            map_location=map_location,
         )  # type: ASRModel
         model_name = cfg.pretrained_name
 
+    if hasattr(cfg, "model_change") and hasattr(asr_model, "change_attention_model"):
+        asr_model.change_attention_model(
+            self_attention_model=cfg.model_change.conformer.get("self_attention_model", None),
+            att_context_size=cfg.model_change.conformer.get("att_context_size", None),
+        )
+
     return asr_model, model_name
 
 
 def prepare_audio_data(cfg: DictConfig) -> Tuple[List[str], bool]:
-    """ Prepare audio data and decide whether it's partial_audio condition. """
-    # this part may need refactor alongsides with refactor of transcribe
-    partial_audio = False
+    """
+    Prepare audio data for transcription.
+    Args:
+        cfg (DictConfig): Configuration dictionary containing the following parameters:
+            - audio_dir (str): Path to the directory containing audio files.
+            - append_pred (bool): Flag indicating whether to append predictions to an existing dataset.
+            - audio_type (str): Type of audio files to consider.
+            - dataset_manifest (str): Path to the dataset manifest file.
+            - audio_key (str, optional): Key in the manifest file specifying the audio file path.
+                Defaults to 'audio_filepath'.
+            - presort_manifest (bool, optional): Flag indicating whether to presort the manifest file.
+                Defaults to True.
+    Returns:
+        Tuple[List[str], bool]: A tuple containing the following:
+            - filepaths (List[str]): List of filepaths to the audio files if path to the directory
+                containing audio files is provided.
+            - sorted_manifest_path (bool): Path to the sorted manifest file if path to the dataset
+                manifest file is provided.
+    """
+
+    filepaths = None
+    sorted_manifest_path = None
 
     if cfg.audio_dir is not None and not cfg.append_pred:
         filepaths = list(glob.glob(os.path.join(cfg.audio_dir, f"**/*.{cfg.audio_type}"), recursive=True))
     else:
-        # get filenames from manifest
         filepaths = []
         if os.stat(cfg.dataset_manifest).st_size == 0:
             logging.error(f"The input dataset_manifest {cfg.dataset_manifest} is empty. Exiting!")
             return None
 
-        with open(cfg.dataset_manifest, 'r', encoding='utf_8') as f:
-            has_two_fields = []
-            for line in f:
-                item = json.loads(line)
-                if "offset" in item and "duration" in item:
-                    has_two_fields.append(True)
-                else:
-                    has_two_fields.append(False)
-                audio_key = cfg.get('audio_key', 'audio_filepath')
-                audio_file = get_full_path(audio_file=item[audio_key], manifest_file=cfg.dataset_manifest)
-                filepaths.append(audio_file)
-        partial_audio = all(has_two_fields)
-    logging.info(f"\nTranscribing {len(filepaths)} files...\n")
+        audio_key = cfg.get('audio_key', 'audio_filepath')
 
-    return filepaths, partial_audio
+        with open(cfg.dataset_manifest, "rt") as fh:
+            for line in fh:
+                line = line.strip()
+                if not line:
+                    continue
+                item = json.loads(line)
+                item[audio_key] = get_full_path(item[audio_key], cfg.dataset_manifest)
+                if item.get("duration") is None and cfg.presort_manifest:
+                    raise ValueError(
+                        f"Requested presort_manifest=True, but line {line} in manifest {cfg.dataset_manifest} \
+                            lacks a 'duration' field."
+                    )
+
+        with NamedTemporaryFile(mode='w', suffix='.json', delete=False) as f:
+            for item in read_and_maybe_sort_manifest(cfg.dataset_manifest, try_sort=cfg.presort_manifest):
+                audio_file = get_full_path(audio_file=item[audio_key], manifest_file=cfg.dataset_manifest)
+                item['audio_filepath'] = audio_file
+                filepaths.append(audio_file)
+                f.write(json.dumps(item) + "\n")
+        sorted_manifest_path = f.name
+
+    return filepaths, sorted_manifest_path
+
+
+def read_and_maybe_sort_manifest(path: str, try_sort: bool = False) -> List[dict]:
+    """Sorts the manifest if duration key is available for every utterance."""
+    items = manifest_utils.read_manifest(path)
+    if try_sort and all("duration" in item and item["duration"] is not None for item in items):
+        items = sorted(items, reverse=True, key=lambda item: item["duration"])
+    return items
+
+
+def restore_transcription_order(manifest_path: str, transcriptions: list) -> list:
+    with open(manifest_path) as f:
+        items = [(idx, json.loads(l)) for idx, l in enumerate(f) if l.strip() != ""]
+    if not all("duration" in item[1] and item[1]["duration"] is not None for item in items):
+        return transcriptions
+    new2old = [item[0] for item in sorted(items, reverse=True, key=lambda it: it[1]["duration"])]
+    del items  # free up some memory
+    is_list = isinstance(transcriptions[0], list)
+    if is_list:
+        transcriptions = list(zip(*transcriptions))
+    reordered = [None] * len(transcriptions)
+    for new, old in enumerate(new2old):
+        reordered[old] = transcriptions[new]
+    if is_list:
+        reordered = tuple(map(list, zip(*reordered)))
+    return reordered
 
 
 def compute_output_filename(cfg: DictConfig, model_name: str) -> DictConfig:
-    """ Compute filename of output manifest and update cfg"""
+    """Compute filename of output manifest and update cfg"""
     if cfg.output_filename is None:
         # create default output filename
         if cfg.audio_dir is not None:
@@ -272,14 +410,14 @@ def normalize_timestamp_output(timestamps: dict):
 
 
 def write_transcription(
-    transcriptions: Union[List[rnnt_utils.Hypothesis], List[List[rnnt_utils.Hypothesis]]],
+    transcriptions: Union[List[rnnt_utils.Hypothesis], List[List[rnnt_utils.Hypothesis]], List[str]],
     cfg: DictConfig,
     model_name: str,
     filepaths: List[str] = None,
     compute_langs: bool = False,
-    compute_timestamps: bool = False,
+    timestamps: bool = False,
 ) -> Tuple[str, str]:
-    """ Write generated transcription to output file. """
+    """Write generated transcription to output file."""
     if cfg.append_pred:
         logging.info(f'Transcripts will be written in "{cfg.output_filename}" file')
         if cfg.pred_name_postfix is not None:
@@ -290,7 +428,11 @@ def write_transcription(
     else:
         pred_text_attr_name = 'pred_text'
 
-    if isinstance(transcriptions[0], rnnt_utils.Hypothesis):  # List[rnnt_utils.Hypothesis]
+    return_hypotheses = True
+    if isinstance(transcriptions[0], str):  # List[str]:
+        best_hyps = transcriptions
+        return_hypotheses = False
+    elif isinstance(transcriptions[0], rnnt_utils.Hypothesis):  # List[rnnt_utils.Hypothesis]
         best_hyps = transcriptions
         assert cfg.decoding.beam.return_best_hypothesis, "Works only with return_best_hypothesis=true"
     elif isinstance(transcriptions[0], list) and isinstance(
@@ -302,158 +444,162 @@ def write_transcription(
             if not cfg.decoding.beam.return_best_hypothesis:
                 beam = []
                 for hyp in hyps:
-                    beam.append((hyp.text, hyp.score))
+                    score = hyp.score.numpy().item() if isinstance(hyp.score, torch.Tensor) else hyp.score
+                    beam.append((hyp.text, score))
                 beams.append(beam)
     else:
         raise TypeError
 
+    # create output dir if not exists
+    Path(cfg.output_filename).parent.mkdir(parents=True, exist_ok=True)
     with open(cfg.output_filename, 'w', encoding='utf-8', newline='\n') as f:
         if cfg.audio_dir is not None:
-            for idx, transcription in enumerate(best_hyps):  # type: rnnt_utils.Hypothesis
-                item = {'audio_filepath': filepaths[idx], pred_text_attr_name: transcription.text}
+            for idx, transcription in enumerate(best_hyps):  # type: rnnt_utils.Hypothesis or str
+                if not return_hypotheses:  # transcription is str
+                    item = {'audio_filepath': filepaths[idx], pred_text_attr_name: transcription}
+                else:  # transcription is Hypothesis
+                    item = {'audio_filepath': filepaths[idx], pred_text_attr_name: transcription.text}
 
-                if compute_timestamps:
-                    timestamps = transcription.timestep
-                    if timestamps is not None and isinstance(timestamps, dict):
-                        timestamps.pop('timestep', None)  # Pytorch tensor calculating index of each token, not needed.
-                        for key in timestamps.keys():
-                            values = normalize_timestamp_output(timestamps[key])
-                            item[f'timestamps_{key}'] = values
-
-                if compute_langs:
-                    item['pred_lang'] = transcription.langs
-                    item['pred_lang_chars'] = transcription.langs_chars
-                if not cfg.decoding.beam.return_best_hypothesis:
-                    item['beams'] = beams[idx]
-                f.write(json.dumps(item) + "\n")
-        else:
-            with open(cfg.dataset_manifest, 'r', encoding='utf-8') as fr:
-                for idx, line in enumerate(fr):
-                    item = json.loads(line)
-                    item[pred_text_attr_name] = best_hyps[idx].text
-
-                    if compute_timestamps:
-                        timestamps = best_hyps[idx].timestep
+                    if timestamps:
+                        timestamps = transcription.timestamp
                         if timestamps is not None and isinstance(timestamps, dict):
                             timestamps.pop(
                                 'timestep', None
                             )  # Pytorch tensor calculating index of each token, not needed.
                             for key in timestamps.keys():
                                 values = normalize_timestamp_output(timestamps[key])
-                                item[f'timestamps_{key}'] = values
+                                item[f'{key}'] = values
 
                     if compute_langs:
-                        item['pred_lang'] = best_hyps[idx].langs
-                        item['pred_lang_chars'] = best_hyps[idx].langs_chars
-
+                        item['pred_lang'] = transcription.langs
+                        item['pred_lang_chars'] = transcription.langs_chars
                     if not cfg.decoding.beam.return_best_hypothesis:
                         item['beams'] = beams[idx]
+                f.write(json.dumps(item) + "\n")
+        else:
+            with open(cfg.dataset_manifest, 'r', encoding='utf-8') as fr:
+                for idx, line in enumerate(fr):
+                    line = line.strip()
+                    if not line:
+                        continue
+                    item = json.loads(line)
+                    if not return_hypotheses:  # transcription is str
+                        item[pred_text_attr_name] = best_hyps[idx]
+                    else:  # transcription is Hypothesis
+                        item[pred_text_attr_name] = best_hyps[idx].text
+
+                        if timestamps:
+                            timestamps = best_hyps[idx].timestamp
+                            if timestamps is not None and isinstance(timestamps, dict):
+                                timestamps.pop(
+                                    'timestep', None
+                                )  # Pytorch tensor calculating index of each token, not needed.
+                                for key in timestamps.keys():
+                                    values = normalize_timestamp_output(timestamps[key])
+                                    item[f'{key}'] = values
+
+                        if compute_langs:
+                            item['pred_lang'] = best_hyps[idx].langs
+                            item['pred_lang_chars'] = best_hyps[idx].langs_chars
+
+                        if not cfg.decoding.beam.return_best_hypothesis:
+                            item['beams'] = beams[idx]
                     f.write(json.dumps(item) + "\n")
 
     return cfg.output_filename, pred_text_attr_name
 
 
-def transcribe_partial_audio(
-    asr_model,
-    path2manifest: str = None,
-    batch_size: int = 4,
-    logprobs: bool = False,
-    return_hypotheses: bool = False,
-    num_workers: int = 0,
-    channel_selector: Optional[int] = None,
-    augmentor: DictConfig = None,
-    decoder_type: Optional[str] = None,
-) -> List[str]:
-    """
-    See description of this function in trancribe() in nemo/collections/asr/models/ctc_models.py and nemo/collections/asr/models/rnnt_models.py
-    """
+def compute_metrics_per_sample(
+    manifest_path: str,
+    reference_field: str = "text",
+    hypothesis_field: str = "pred_text",
+    metrics: List[str] = ["wer"],
+    punctuation_marks: List[str] = [".", ",", "?"],
+    output_manifest_path: str = None,
+) -> dict:
+    '''
+    Computes metrics per sample for given manifest
 
-    if return_hypotheses and logprobs:
-        raise ValueError(
-            "Either `return_hypotheses` or `logprobs` can be True at any given time."
-            "Returned hypotheses will contain the logprobs."
+    Args:
+        manifest_path: str, Required - path to dataset JSON manifest file (in NeMo format)
+        reference_field: str, Optional - name of field in .json manifest with the reference text
+            ("text" by default).
+        hypothesis_field: str, Optional - name of field in .json manifest with the hypothesis text
+            ("pred_text" by default).
+        metrics: list[str], Optional - list of metrics to be computed
+            (currently supported "wer", "cer", "punct_er")
+        punctuation_marks: list[str], Optional - list of punctuation marks for computing
+            punctuation error rate ([".", ",", "?"] by default).
+        output_manifest_path: str, Optional - path where .json manifest with calculated metrics will be saved.
+
+    Returns:
+        samples: dict - Dict of samples with calculated metrics
+    '''
+
+    supported_metrics = ["wer", "cer", "punct_er"]
+
+    if len(metrics) == 0:
+        raise AssertionError(
+            f"'metrics' list is empty. \
+            Select the metrics from the supported: {supported_metrics}."
         )
-    if num_workers is None:
-        num_workers = min(batch_size, os.cpu_count() - 1)
 
-    # We will store transcriptions here
-    hypotheses = []
-    # Model's mode and device
-    mode = asr_model.training
-    device = next(asr_model.parameters()).device
-    dither_value = asr_model.preprocessor.featurizer.dither
-    pad_to_value = asr_model.preprocessor.featurizer.pad_to
-
-    if decoder_type is not None:  # Hybrid model
-        decode_function = (
-            asr_model.decoding.rnnt_decoder_predictions_tensor
-            if decoder_type == 'rnnt'
-            else asr_model.decoding.ctc_decoder_predictions_tensor
-        )
-    elif hasattr(asr_model, 'joint'):  # RNNT model
-        decode_function = asr_model.decoding.rnnt_decoder_predictions_tensor
-    else:  # CTC model
-        decode_function = asr_model.decoding.ctc_decoder_predictions_tensor
-
-    try:
-        asr_model.preprocessor.featurizer.dither = 0.0
-        asr_model.preprocessor.featurizer.pad_to = 0
-        # Switch model to evaluation mode
-        asr_model.eval()
-        # Freeze the encoder and decoder modules
-        asr_model.encoder.freeze()
-        asr_model.decoder.freeze()
-        logging_level = logging.get_verbosity()
-        logging.set_verbosity(logging.WARNING)
-
-        config = {
-            'manifest_filepath': path2manifest,
-            'batch_size': batch_size,
-            'num_workers': num_workers,
-            'channel_selector': channel_selector,
-        }
-        if augmentor:
-            config['augmentor'] = augmentor
-
-        temporary_datalayer = asr_model._setup_transcribe_dataloader(config)
-        for test_batch in tqdm(temporary_datalayer, desc="Transcribing"):
-            outputs = asr_model.forward(
-                input_signal=test_batch[0].to(device), input_signal_length=test_batch[1].to(device)
+    for metric in metrics:
+        if metric not in supported_metrics:
+            raise AssertionError(
+                f"'{metric}' metric is not supported. \
+                Currently supported metrics are {supported_metrics}."
             )
-            logits, logits_len = outputs[0], outputs[1]
-            if logprobs:
-                # dump log probs per file
-                for idx in range(logits.shape[0]):
-                    lg = logits[idx][: logits_len[idx]]
-                    hypotheses.append(lg.cpu().numpy())
-            else:
-                current_hypotheses, all_hyp = decode_function(logits, logits_len, return_hypotheses=return_hypotheses,)
 
-                if isinstance(current_hypotheses, tuple) and len(current_hypotheses) == 2:
-                    current_hypotheses = current_hypotheses[0]
+    if "punct_er" in metrics:
+        if len(punctuation_marks) == 0:
+            raise AssertionError("punctuation_marks list can't be empty when 'punct_er' metric is enabled.")
+        else:
+            oper_obj = OccurancePunctuationErrorRate(punctuation_marks=punctuation_marks)
 
-                if return_hypotheses:
-                    # dump log probs per file
-                    for idx in range(logits.shape[0]):
-                        current_hypotheses[idx].y_sequence = logits[idx][: logits_len[idx]]
-                        if current_hypotheses[idx].alignments is None:
-                            current_hypotheses[idx].alignments = current_hypotheses[idx].y_sequence
+    use_wer = "wer" in metrics
+    use_cer = "cer" in metrics
+    use_punct_er = "punct_er" in metrics
 
-                hypotheses += current_hypotheses
+    with open(manifest_path, 'r') as manifest:
+        lines = manifest.readlines()
+        samples = [json.loads(line) for line in lines if line.strip() != ""]
+        samples_with_metrics = []
 
-            del logits
-            del test_batch
+        logging.info(f"Computing {', '.join(metrics)} per sample")
 
-    finally:
-        # set mode back to its original value
-        asr_model.train(mode=mode)
-        asr_model.preprocessor.featurizer.dither = dither_value
-        asr_model.preprocessor.featurizer.pad_to = pad_to_value
-        if mode is True:
-            asr_model.encoder.unfreeze()
-            asr_model.decoder.unfreeze()
-        logging.set_verbosity(logging_level)
-    return hypotheses
+        for sample in tqdm(samples):
+            reference = sample[reference_field]
+            hypothesis = sample[hypothesis_field]
+
+            if use_wer:
+                sample_wer = word_error_rate(hypotheses=[hypothesis], references=[reference], use_cer=False)
+                sample["wer"] = round(100 * sample_wer, 2)
+
+            if use_cer:
+                sample_cer = word_error_rate(hypotheses=[hypothesis], references=[reference], use_cer=True)
+                sample["cer"] = round(100 * sample_cer, 2)
+
+            if use_punct_er:
+                operation_amounts, substitution_amounts, punctuation_rates = oper_obj.compute(
+                    reference=reference, hypothesis=hypothesis
+                )
+                sample["punct_correct_rate"] = round(100 * punctuation_rates.correct_rate, 2)
+                sample["punct_deletions_rate"] = round(100 * punctuation_rates.deletions_rate, 2)
+                sample["punct_insertions_rate"] = round(100 * punctuation_rates.insertions_rate, 2)
+                sample["punct_substitutions_rate"] = round(100 * punctuation_rates.substitutions_rate, 2)
+                sample["punct_error_rate"] = round(100 * punctuation_rates.punct_er, 2)
+
+            samples_with_metrics.append(sample)
+
+    if output_manifest_path is not None:
+        with open(output_manifest_path, 'w') as output:
+            for sample in samples_with_metrics:
+                line = json.dumps(sample)
+                output.writelines(f'{line}\n')
+        logging.info(f'Output manifest saved: {output_manifest_path}')
+
+    return samples_with_metrics
 
 
 class PunctuationCapitalization:
@@ -467,7 +613,7 @@ class PunctuationCapitalization:
         """
         if punctuation_marks:
             self.regex_punctuation = re.compile(fr"([{''.join(punctuation_marks)}])")
-            self.regex_extra_space = re.compile('\s{2,}')
+            self.regex_extra_space = re.compile(r'\s{2,}')
         else:
             self.regex_punctuation = None
 
