@@ -14,11 +14,14 @@
 
 import json
 import os
+import re
+from collections import defaultdict
 from os.path import expanduser
 from typing import Any, Callable, Dict, Iterator, List, Optional, Union
 
 from nemo.utils import logging
-from nemo.utils.data_utils import DataStoreObject, datastore_path_to_local_path, is_datastore_path
+from nemo.utils.data_utils import DataStoreObject, get_datastore_object, is_datastore_path
+from nemo.utils.nemo_logging import LogMode
 
 
 class ManifestBase:
@@ -68,6 +71,7 @@ def item_iter(
     if parse_func is None:
         parse_func = __parse_item
 
+    errors = defaultdict(list)
     k = -1
     logging.debug('Manifest files: %s', str(manifests_files))
     for manifest_file in manifests_files:
@@ -76,12 +80,27 @@ def item_iter(
         logging.debug('Cached at: %s', str(cached_manifest_file))
         with open(expanduser(cached_manifest_file), 'r') as f:
             for line in f:
+                line = line.strip()
+                if not line:
+                    continue
                 k += 1
-                item = parse_func(line, manifest_file)
+                try:
+                    item = parse_func(line, manifest_file)
+                except json.JSONDecodeError:
+                    errors[str(manifest_file)].append(line)
+                    continue
                 item['id'] = k
 
                 yield item
 
+    if len(errors) > 0:
+        for filename, lines in errors.items():
+            logging.error("=============================================")
+            logging.error(f"Failed to parse {len(lines)} lines from manifest file: {filename}")
+            for line in lines:
+                logging.error(f"-- Failed to parse line: `{line}`")
+        raise RuntimeError("Failed to parse some lines from manifest files. See logs for more details.")
+
 
 def __parse_item(line: str, manifest_file: str) -> Dict[str, Any]:
     item = json.loads(line)
@@ -91,19 +110,33 @@ def __parse_item(line: str, manifest_file: str) -> Dict[str, Any]:
         item['audio_file'] = item.pop('audio_filename')
     elif 'audio_filepath' in item:
         item['audio_file'] = item.pop('audio_filepath')
-    elif 'audio_file' not in item:
+    elif 'context' in item:
+        item['audio_file'] = item['context']
+
+    # Video File
+    if 'video_filename' in item:
+        item['video_file'] = item.pop('video_filename')
+    elif 'video_filepath' in item:
+        item['video_file'] = item.pop('video_filepath')
+
+    if 'video_file' not in item and 'audio_file' not in item:
         raise ValueError(
-            f"Manifest file {manifest_file} has invalid json line structure: {line} without proper audio file key."
+            f"Manifest file {manifest_file} has invalid json line structure: {line} without proper audio/video file key."
         )
 
-    # If the audio path is a relative path and does not exist,
+    # If the audio/video path is a relative path and does not exist,
     # try to attach the parent directory of manifest to the audio path.
     # Revert to the original path if the new path still doesn't exist.
     # Assume that the audio path is like "wavs/xxxxxx.wav".
-    item['audio_file'] = get_full_path(audio_file=item['audio_file'], manifest_file=manifest_file)
+    if 'audio_file' in item:
+        item['audio_file'] = get_full_path(audio_file=item['audio_file'], manifest_file=manifest_file)
+    if 'video_file' in item:
+        item['video_file'] = get_full_path(audio_file=item['video_file'], manifest_file=manifest_file)
 
     # Duration.
-    if 'duration' not in item:
+    if 'context_duration' in item and 'duration' not in item:
+        item['duration'] = item['context_duration']
+    elif 'duration' not in item:
         raise ValueError(
             f"Manifest file {manifest_file} has invalid json line structure: {line} without proper duration key."
         )
@@ -144,7 +177,8 @@ def __parse_item(line: str, manifest_file: str) -> Dict[str, Any]:
         item['feature_file'] = get_full_path(audio_file=item['feature_file'], manifest_file=manifest_file)
 
     item = dict(
-        audio_file=item['audio_file'],
+        audio_file=item.get('audio_file', None),
+        video_file=item.get('video_file', None),
         duration=item['duration'],
         text=item['text'],
         rttm_file=item['rttm_file'],
@@ -154,15 +188,38 @@ def __parse_item(line: str, manifest_file: str) -> Dict[str, Any]:
         orig_sr=item.get('orig_sample_rate', None),
         token_labels=item.get('token_labels', None),
         lang=item.get('lang', None),
+        context=item.get('context', None),
+        context_type=item.get('context_type', None),
+        context_duration=item.get('context_duration', None),
+        answer=item.get('answer', None),
+        answer_type=item.get('answer_type', None),
+        answer_duration=item.get('answer_duration', None),
+        question=item.get('question', None),
+        question_type=item.get('question_type', None),
+        task=item.get('task', None),
     )
     return item
 
 
+def is_tarred_dataset(audio_file: str, manifest_file: Optional[str] = None) -> bool:
+    if "/" in audio_file or manifest_file is None:
+        # audio files in a tarred dataset don't have `/` in their paths
+        return False
+    if os.path.basename(manifest_file) == "tarred_audio_manifest.json":
+        # the manifest file is a tarred manifest
+        return True
+    if "/sharded_manifests/" in manifest_file and re.match(r'^manifest_(\d+)\.json$', os.path.basename(manifest_file)):
+        # the manifest file is a sharded manifest
+        return True
+    return False
+
+
 def get_full_path(
     audio_file: Union[str, List[str]],
     manifest_file: Optional[str] = None,
     data_dir: Optional[str] = None,
     audio_file_len_limit: int = 255,
+    force_cache: bool = True,
 ) -> Union[str, List[str]]:
     """Get full path to audio_file.
 
@@ -178,6 +235,7 @@ def get_full_path(
         manifest_file: path to a manifest file
         data_dir: path to a directory containing data, use only if a manifest file is not provided
         audio_file_len_limit: limit for length of audio_file when using relative paths
+        force_cache: cache and provide local cached path of the audio if audio_file is from a data object store
 
     Returns:
         Full path to audio_file or a list of paths.
@@ -190,20 +248,27 @@ def get_full_path(
                 manifest_file=manifest_file,
                 data_dir=data_dir,
                 audio_file_len_limit=audio_file_len_limit,
+                force_cache=force_cache,
             )
             for a_file in audio_file
         ]
     elif isinstance(audio_file, str):
         # If input is a string, get the corresponding full path
+        if is_tarred_dataset(audio_file=audio_file, manifest_file=manifest_file):
+            logging.warning(
+                f"Manifest file `{manifest_file}` seems to be part of a tarred dataset, skip checking for relative paths. If this is not intended, please avoid having `/sharded_manifests/` and `tarred_audio_manifest.json` in manifest_filepath.",
+                mode=LogMode.ONCE,
+            )
+            return audio_file
         if (
             (len(audio_file) < audio_file_len_limit)
             and not os.path.isabs(audio_file)
-            and not os.path.isfile(audio_file)
+            # and not os.path.isfile(audio_file) # Commented out because it slows down dataloading
         ):
             # If audio_file is not available and the path is not absolute, the full path is assumed
             # to be relative to the manifest file parent directory or data directory.
             if manifest_file is None and data_dir is None:
-                raise ValueError(f'Use either manifest_file or data_dir to specify the data directory.')
+                raise ValueError('Use either manifest_file or data_dir to specify the data directory.')
             elif manifest_file is not None and data_dir is not None:
                 raise ValueError(
                     f'Parameters manifest_file and data_dir cannot be used simultaneously. Currently manifest_file is {manifest_file} and data_dir is {data_dir}.'
@@ -217,8 +282,10 @@ def get_full_path(
             audio_file_path = os.path.join(data_dir, audio_file)
 
             if is_datastore_path(audio_file_path):
-                # If audio was originally on an object store, use locally-cached path
-                audio_file_path = datastore_path_to_local_path(audio_file_path)
+                # If audio was originally on an object store, use locally-cached path.
+                # If the file could not be found in the local cache, it will be downloaded.
+                audio_file_path = get_datastore_object(audio_file_path) if force_cache else audio_file_path
+                return audio_file_path
 
             if os.path.isfile(audio_file_path):
                 audio_file = os.path.abspath(audio_file_path)
