@@ -13,11 +13,11 @@
 # limitations under the License.
 
 import torch
-from pytorch_lightning.plugins.precision.native_amp import NativeMixedPrecisionPlugin
+from lightning.pytorch.plugins.precision import MixedPrecisionPlugin
 
 
-class NeMoNativeMixedPrecisionPlugin(NativeMixedPrecisionPlugin):
-    def __init__(self, init_scale: float = 2 ** 32, growth_interval: int = 1000) -> None:
+class NeMoMixedPrecisionPlugin(MixedPrecisionPlugin):
+    def __init__(self, init_scale: float = 2**32, growth_interval: int = 1000) -> None:
         super().__init__(precision=16)
 
         self.scaler = torch.cuda.amp.GradScaler(init_scale=init_scale, growth_interval=growth_interval)
