@@ -22,8 +22,8 @@ import os
 from typing import List, Optional
 
 import torch
+from lightning.pytorch import Trainer
 from omegaconf import DictConfig
-from pytorch_lightning import Trainer
 from torch.utils.data import DataLoader
 
 from nemo.collections.nlp.data.dialogue import DialogueSGDBERTDataset, DialogueSGDDataProcessor
@@ -35,6 +35,7 @@ from nemo.collections.nlp.modules import SGDDecoder, SGDEncoder
 from nemo.collections.nlp.parts.utils_funcs import tensor2list
 from nemo.core.classes.common import PretrainedModelInfo, typecheck
 from nemo.utils import logging
+from nemo.utils.decorators import deprecated_warning
 
 __all__ = ['SGDQAModel']
 
@@ -44,7 +45,7 @@ class SGDQAModel(NLPModel):
     Dialogue State Tracking Model SGD-QA (https://arxiv.org/abs/2105.08049)
 
     The SGD-QA model is a fast multi-pass schema-guided state-tracking model, that is trained on the Google schema-guided state tracking dataset (https://arxiv.org/abs/1909.05855).
-    The model takes dialogue as input and outputs the dialogue state, which includes slot-value pairs. 
+    The model takes dialogue as input and outputs the dialogue state, which includes slot-value pairs.
     The model consists of two components: a neural natural language understanding model (NLU), and a rule-based state tracker.
     The NLU takes in a dialogue turn and different schema (entity) information options and outputs their match score. The state tracker takes the highest rated entities and composes
     the dialogue state across turns.
@@ -55,6 +56,9 @@ class SGDQAModel(NLPModel):
         return self.decoder
 
     def __init__(self, cfg: DictConfig, trainer: Trainer = None):
+        # deprecation warning
+        deprecated_warning("SGDQAModel")
+
         self.data_prepared = False
         super().__init__(cfg=cfg, trainer=trainer)
         self.encoder = SGDEncoder(hidden_size=self.bert_model.config.hidden_size, dropout=self._cfg.encoder.dropout)
@@ -146,11 +150,16 @@ class SGDQAModel(NLPModel):
         Called at every validation step to aggregate and postprocess outputs on each GPU
         Args:
             batch: input batch at validation step
-            batch_idx: batch index 
+            batch_idx: batch index
             dataloader_idx: dataloader index
         """
         loss, tensors = self.eval_step_helper(batch=batch)
         self.log(f'val_loss', loss)
+        if type(self.trainer.val_dataloaders) == list and len(self.trainer.val_dataloaders) > 1:
+            self.validation_step_outputs[dataloader_idx].append({f'val_loss': loss, f'tensors': tensors})
+        else:
+            self.validation_step_outputs.append({f'val_loss': loss, f'tensors': tensors})
+
         return {f'val_loss': loss, f'tensors': tensors}
 
     def test_step(self, batch: List[torch.Tensor], batch_idx: int, dataloader_idx: int = 0) -> dict:
@@ -158,10 +167,15 @@ class SGDQAModel(NLPModel):
         Called at every test step to aggregate and postprocess outputs on each GPU
         Args:
             batch: input batch at test step
-            batch_idx: batch index 
+            batch_idx: batch index
             dataloader_idx: dataloader index
         """
         loss, tensors = self.eval_step_helper(batch=batch)
+        if type(self.trainer.test_dataloaders) == list and len(self.trainer.test_dataloaders) > 1:
+            self.test_step_outputs[dataloader_idx].append({f'test_loss': loss, f'tensors': tensors})
+        else:
+            self.test_step_outputs.append({f'test_loss': loss, f'tensors': tensors})
+
         return {f'test_loss': loss, f'tensors': tensors}
 
     def eval_step_helper(self, batch: List[torch.Tensor]):
@@ -308,8 +322,8 @@ class SGDQAModel(NLPModel):
             torch.zeros(total_scores.size(), device=total_scores.get_device(), dtype=total_scores.dtype),
             total_scores,
         )
-        max_span_index = torch.argmax(total_scores.view(-1, max_num_tokens ** 2), axis=-1)
-        max_span_p = torch.max(total_scores.view(-1, max_num_tokens ** 2), axis=-1)[0]
+        max_span_index = torch.argmax(total_scores.view(-1, max_num_tokens**2), axis=-1)
+        max_span_p = torch.max(total_scores.view(-1, max_num_tokens**2), axis=-1)[0]
 
         span_start_index = torch.floor_divide(max_span_index, max_num_tokens)
         span_end_index = torch.fmod(max_span_index, max_num_tokens)
@@ -405,7 +419,7 @@ class SGDQAModel(NLPModel):
 
         def combine_predictions_in_example(predictions: dict, batch_size: int):
             '''
-            Combines predicted values to a single example. 
+            Combines predicted values to a single example.
             Args:
                 predictions: predictions ordered by keys then batch
                 batch_size: batch size
