@@ -18,8 +18,8 @@ from typing import List, Optional, Tuple
 import numpy as np
 import numpy.typing as npt
 import torch
+from lightning.pytorch import Trainer
 from omegaconf import DictConfig, OmegaConf
-from pytorch_lightning import Trainer
 from sklearn.metrics import f1_score, precision_score, recall_score
 from torch.utils.data import DataLoader
 
@@ -38,10 +38,10 @@ from nemo.utils import logging
 
 class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
     def __init__(self, cfg: DictConfig, trainer: Trainer = None):
-        """ 
+        """
         Initializes BERT Joint Intent and Slot model.
 
-        Args: 
+        Args:
             cfg: configuration object
             trainer: trainer for Pytorch Lightning
         """
@@ -69,12 +69,12 @@ class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
     def _set_data_desc_to_cfg(
         self, cfg: DictConfig, data_dir: str, train_ds: DictConfig, validation_ds: DictConfig
     ) -> None:
-        """ 
-        Creates MultiLabelIntentSlotDataDesc and copies generated values to Configuration object's data descriptor. 
-        
-        Args: 
+        """
+        Creates MultiLabelIntentSlotDataDesc and copies generated values to Configuration object's data descriptor.
+
+        Args:
             cfg: configuration object
-            data_dir: data directory 
+            data_dir: data directory
             train_ds: training dataset file name
             validation_ds: validation dataset file name
 
@@ -101,7 +101,10 @@ class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
         if not hasattr(cfg, "class_labels") or cfg.class_labels is None:
             cfg.class_labels = {}
             cfg.class_labels = OmegaConf.create(
-                {"intent_labels_file": "intent_labels.csv", "slot_labels_file": "slot_labels.csv",}
+                {
+                    "intent_labels_file": "intent_labels.csv",
+                    "slot_labels_file": "slot_labels.csv",
+                }
             )
 
         slot_labels_file = os.path.join(data_dir, cfg.class_labels.slot_labels_file)
@@ -114,7 +117,7 @@ class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
         OmegaConf.set_struct(cfg, True)
 
     def _reconfigure_classifier(self) -> None:
-        """ Method reconfigures the classifier depending on the settings of model cfg.data_desc """
+        """Method reconfigures the classifier depending on the settings of model cfg.data_desc"""
 
         self.classifier = SequenceTokenClassifier(
             hidden_size=self.bert_model.config.hidden_size,
@@ -135,7 +138,8 @@ class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
             self.slot_loss = CrossEntropyLoss(logits_ndim=3)
 
         self.total_loss = AggregatorLoss(
-            num_inputs=2, weights=[self.cfg.intent_loss_weight, 1.0 - self.cfg.intent_loss_weight],
+            num_inputs=2,
+            weights=[self.cfg.intent_loss_weight, 1.0 - self.cfg.intent_loss_weight],
         )
 
         # setup to track metrics
@@ -161,12 +165,22 @@ class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
             batch: batches of data from DataLoader
             batch_idx: batch idx from DataLoader
 
-        Returns: 
+        Returns:
             None
         """
-        (input_ids, input_type_ids, input_mask, loss_mask, subtokens_mask, intent_labels, slot_labels,) = batch
+        (
+            input_ids,
+            input_type_ids,
+            input_mask,
+            loss_mask,
+            subtokens_mask,
+            intent_labels,
+            slot_labels,
+        ) = batch
         intent_logits, slot_logits = self(
-            input_ids=input_ids, token_type_ids=input_type_ids, attention_mask=input_mask,
+            input_ids=input_ids,
+            token_type_ids=input_type_ids,
+            attention_mask=input_mask,
         )
 
         # calculate combined loss for intents and slots
@@ -183,7 +197,7 @@ class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
         slot_labels = slot_labels[subtokens_mask]
         self.slot_classification_report.update(preds, slot_labels)
 
-        return {
+        loss = {
             "val_loss": val_loss,
             "intent_tp": self.intent_classification_report.tp,
             "intent_fn": self.intent_classification_report.fn,
@@ -192,6 +206,8 @@ class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
             "slot_fn": self.slot_classification_report.fn,
             "slot_fp": self.slot_classification_report.fp,
         }
+        self.validation_step_outputs.append(loss)
+        return loss
 
     def _setup_dataloader_from_config(self, cfg: DictConfig) -> DataLoader:
         """
@@ -199,7 +215,7 @@ class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
 
         Args:
             cfg: configuration object
-        
+
         Returns:
             DataLoader for model's data
         """
@@ -287,8 +303,8 @@ class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
 
     def optimize_threshold(self, test_ds: DictConfig, file_name: str) -> None:
         """
-        Set the optimal threshold of the model from performance on validation set. This threshold is used to round the 
-        logits to 0 or 1. 
+        Set the optimal threshold of the model from performance on validation set. This threshold is used to round the
+        logits to 0 or 1.
 
         Args:
             test_ds: location of test dataset
@@ -359,16 +375,16 @@ class MultiLabelIntentSlotClassificationModel(IntentSlotClassificationModel):
             queries: text sequences
             test_ds: Dataset configuration section.
             threshold: Threshold for rounding prediction logits
-        
+
         Returns:
             predicted_intents: model intent predictions with their probabilities
-                Example:  [[('flight', 0.84)], [('airfare', 0.54), 
+                Example:  [[('flight', 0.84)], [('airfare', 0.54),
                             ('flight', 0.73), ('meal', 0.24)]]
             predicted_slots: model slot predictions
                 Example:  ['O B-depart_date.month_name B-depart_date.day_number',
                            'O O B-flight_stop O O O']
 
-            predicted_vector: model intent predictions for each individual query. Binary values within each list 
+            predicted_vector: model intent predictions for each individual query. Binary values within each list
                 indicate whether a class is prediced for the given query (1 for True, 0 for False)
                 Example: [[1,0,0,0,0,0], [0,0,1,0,0,0]]
         """
