@@ -12,20 +12,31 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+# pylint: skip-file
+# flake8: noqa
+
 import os
 from typing import Dict, Optional
 
 import torch
+from lightning.pytorch import Trainer
 from omegaconf import DictConfig, OmegaConf
-from pytorch_lightning import Trainer
 from torch.utils.data import DataLoader
 
 from nemo.collections.common.losses import AggregatorLoss, CrossEntropyLoss, SmoothedCrossEntropyLoss
 from nemo.collections.common.metrics import Perplexity
-from nemo.collections.nlp.data.language_modeling.lm_bert_dataset import (
-    BertPretrainingDataset,
-    BertPretrainingPreprocessedDataloader,
-)
+
+try:
+    from nemo.collections.nlp.data.language_modeling.lm_bert_dataset import (
+        BertPretrainingDataset,
+        BertPretrainingPreprocessedDataloader,
+    )
+except (ImportError, ModuleNotFoundError):
+    from abc import ABC
+
+    BertPretrainingDataset = ABC
+    BertPretrainingPreprocessedDataloader = ABC
+
 from nemo.collections.nlp.modules.common import BertPretrainingTokenClassifier, SequenceClassifier
 from nemo.collections.nlp.modules.common.lm_utils import get_lm_model
 from nemo.collections.nlp.modules.common.tokenizer_utils import get_tokenizer
@@ -75,7 +86,11 @@ class BERTLMModel(ModelPT):
             config_file = self.register_artifact('language_model.config_file', cfg.language_model.config_file)
 
         self.bert_model = get_lm_model(
-            config_file=config_file, config_dict=config_dict, vocab_file=vocab_file, trainer=trainer, cfg=cfg,
+            config_file=config_file,
+            config_dict=config_dict,
+            vocab_file=vocab_file,
+            trainer=trainer,
+            cfg=cfg,
         )
 
         self.hidden_size = self.bert_model.config.hidden_size
@@ -127,7 +142,9 @@ class BERTLMModel(ModelPT):
         in the `nn.Module` in vanilla PyTorch.
         """
         hidden_states = self.bert_model(
-            input_ids=input_ids, token_type_ids=token_type_ids, attention_mask=attention_mask,
+            input_ids=input_ids,
+            token_type_ids=token_type_ids,
+            attention_mask=attention_mask,
         )
         if isinstance(hidden_states, tuple):
             hidden_states = hidden_states[0]
@@ -177,9 +194,11 @@ class BERTLMModel(ModelPT):
         mlm_log_probs, nsp_logits = self._parse_forward_outputs(forward_outputs)
         _, _, loss = self._compute_losses(mlm_log_probs, nsp_logits, output_ids, output_mask, labels)
         self.validation_perplexity(logits=mlm_log_probs)
-        return {'val_loss': loss}
+        loss = {'val_loss': loss}
+        self.validation_step_outputs.append(loss)
+        return loss
 
-    def validation_epoch_end(self, outputs):
+    def on_validation_epoch_end(self):
         """Called at the end of validation to aggregate outputs.
 
         Args:
@@ -188,11 +207,12 @@ class BERTLMModel(ModelPT):
         Returns:
             dict: Validation loss and tensorboard logs.
         """
-        if outputs:
-            avg_loss = torch.stack([x['val_loss'] for x in outputs]).mean()
+        if self.validation_step_outputs:
+            avg_loss = torch.stack([x['val_loss'] for x in self.validation_step_outputs]).mean()
             perplexity = self.validation_perplexity.compute()
             logging.info(f"evaluation perplexity {perplexity.cpu().item()}")
             self.log(f'val_loss', avg_loss)
+            self.validation_step_outputs.clear()  # free memory
 
     def setup_training_data(self, train_data_config: Optional[DictConfig]):
         self._train_dl = (
@@ -222,7 +242,9 @@ class BERTLMModel(ModelPT):
             files = [dataset]
         files.sort()
         dl = BertPretrainingPreprocessedDataloader(
-            data_files=files, max_predictions_per_seq=max_predictions_per_seq, batch_size=batch_size,
+            data_files=files,
+            max_predictions_per_seq=max_predictions_per_seq,
+            batch_size=batch_size,
         )
         return dl
 
