@@ -18,9 +18,9 @@ from functools import partial
 from typing import Any, List, Optional, Union
 
 import torch
+from lightning.pytorch.trainer.trainer import Trainer
 from omegaconf.dictconfig import DictConfig
 from omegaconf.omegaconf import open_dict
-from pytorch_lightning.trainer.trainer import Trainer
 
 from nemo.collections.common.tokenizers.sentencepiece_tokenizer import SentencePieceTokenizer
 from nemo.collections.nlp.data.language_modeling.megatron.gpt_prompt_learning_dataset import GPTPromptLearningDataset
@@ -31,6 +31,7 @@ from nemo.collections.nlp.models.language_modeling.megatron_base_prompt_learning
 from nemo.collections.nlp.models.language_modeling.megatron_gpt_model import MegatronGPTModel
 from nemo.collections.nlp.modules.common import VirtualPromptPlaceholderToken, VirtualPromptSource, VirtualPromptStyle
 from nemo.collections.nlp.modules.common.megatron.utils import (
+    ApexGuardDefaults,
     average_losses_across_data_parallel_group,
     get_iterator_k_split,
 )
@@ -43,56 +44,65 @@ from nemo.collections.nlp.modules.common.transformer.text_generation import Leng
 from nemo.collections.nlp.parts.nlp_overrides import GradScaler, NLPSaveRestoreConnector
 from nemo.collections.nlp.parts.utils_funcs import get_last_rank
 from nemo.utils import AppState, logging
+from nemo.utils.decorators import deprecated_warning
 
 try:
-    from apex.transformer.pipeline_parallel.utils import get_micro_batch_size, get_num_microbatches
-
-    HAVE_APEX = True
-
-except (ImportError, ModuleNotFoundError):
-    HAVE_APEX = False
-
-try:
-    from megatron.core import parallel_state, tensor_parallel
+    from megatron.core import InferenceParams, ModelParallelConfig, parallel_state, tensor_parallel
     from megatron.core.enums import ModelType
     from megatron.core.pipeline_parallel.schedules import get_forward_backward_func
 
     HAVE_MEGATRON_CORE = True
-
 except (ImportError, ModuleNotFoundError):
 
+    ModelParallelConfig = ApexGuardDefaults
+
     HAVE_MEGATRON_CORE = False
 
+try:
+    from megatron.core.num_microbatches_calculator import get_micro_batch_size, get_num_microbatches
+
+except (ImportError, ModuleNotFoundError):
+    logging.warning("Megatron num_microbatches_calculator not found, using Apex version.")
+    from apex.transformer.pipeline_parallel.utils import get_micro_batch_size, get_num_microbatches
+
 
 __all__ = ['MegatronGPTPromptLearningModel']
 
 
 class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
     """
-    Model class for prompt-tuning or p-tuning a pretrained Megatron GPT model. 
+    Model class for prompt-tuning or p-tuning a pretrained Megatron GPT model.
 
     Prompt Tuning initalizes virtual prompt embeddings directly from a copy of
     certain token embeddings from the the pretrained GPT model's vocabulary
-    and directly tunes these embedding weights. The token embeddings used in 
-    initalization are specified by the user in the config file. The model can 
-    be prompt-tuned for multiple tasks at once. virtual prompts are stored in a 
-    prompt table and can be added or deleted without disrupting virtual prompts 
-    for other tasks. 
+    and directly tunes these embedding weights. The token embeddings used in
+    initalization are specified by the user in the config file. The model can
+    be prompt-tuned for multiple tasks at once. virtual prompts are stored in a
+    prompt table and can be added or deleted without disrupting virtual prompts
+    for other tasks.
 
     P-tuning initializes an LSTM encoder model that generates virtual prompt
     embeddings for every task. Each task shares the same encoder. After ptuning
     is compelete, the learned virtual prompts can be saved to the prompt table
-    using add_ptuned_prompts_to_prompt_table(). Thus, if a user wants to add a 
-    new virtual prompt via p-tuning, they do not need to retrain on all previous 
+    using add_ptuned_prompts_to_prompt_table(). Thus, if a user wants to add a
+    new virtual prompt via p-tuning, they do not need to retrain on all previous
     tasks. This gives p-tuning the same task flexiblity as prompt-tuning.
     """
 
     def __init__(self, cfg: DictConfig, trainer: Trainer):
+        # deprecation warning
+        deprecated_warning("MegatronGPTPromptLearningModel")
+
         super().__init__(cfg, trainer)
-        self.init_model(cfg, trainer)
+
+        self.inference_params = None
+
+        # init_model is called by parent class already.
+        # self.init_model(cfg, trainer)
 
     def init_model(self, cfg: DictConfig, trainer: Trainer):
         self.cfg = cfg
+        self.config: ModelParallelConfig = self.model_parallel_config
         save_restore_connector = NLPSaveRestoreConnector()
         if os.path.isdir(cfg.get('language_model_path')):
             save_restore_connector.model_extracted_dir = cfg.get('language_model_path')
@@ -103,6 +113,9 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
             save_restore_connector=save_restore_connector,
         )
 
+        # set hidden size in the model parallel config for pipeline parallel schedules
+        setattr(self.config, 'hidden_size', frozen_model_cfg.hidden_size)
+
         # Need to overwrite some params in frozen model's config before restoring
         with open_dict(frozen_model_cfg):
             frozen_model_cfg.megatron_amp_O2 = False
@@ -119,15 +132,6 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
             )
             frozen_model_cfg.activations_checkpoint_method = self.cfg.get("activations_checkpoint_method", None)
 
-        if self.trainer.precision == 'bf16':
-            self.autocast_dtype = torch.bfloat16
-        elif int(self.trainer.precision) == 32:
-            self.autocast_dtype = torch.float
-        elif int(self.trainer.precision) == 16:
-            self.autocast_dtype = torch.half
-        else:
-            raise ValueError('precision must be in [32, 16, "bf16"]')
-
         if cfg.get('language_model_path', None):
             self.frozen_model = MegatronGPTModel.restore_from(
                 cfg.get('language_model_path'),
@@ -136,7 +140,7 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
                 override_config_path=frozen_model_cfg,
             ).to(dtype=self.autocast_dtype)
 
-        self.megatron_amp_o2 = self.cfg.get('megatron_amp_O2', False)
+        self.megatron_amp_O2 = self.cfg.get('megatron_amp_O2', False)
         self.pipeline_parallel = self.cfg.get('pipeline_model_parallel_size', 1) > 1
         self.tokenizer = self.frozen_model.tokenizer
         self.hidden_size = self.frozen_model.cfg.hidden_size
@@ -145,13 +149,13 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         with open_dict(self.cfg):
             self.cfg.existing_tasks = (
                 self.existing_tasks + self.new_tasks
-            )  # TODO: for backward compatibility (@adithyare) in general these tasks lists should be depricated
+            )  # TODO: for backward compatibility (@adithyare) in general these tasks lists should be deprecated
 
         self.virtual_prompt_style = VirtualPromptStyle(cfg.virtual_prompt_style)
         self.model_type = ModelType.encoder_or_decoder
 
         self.enable_autocast = (
-            True if (not self.megatron_amp_o2) and (self.autocast_dtype in [torch.float16, torch.bfloat16]) else False
+            True if (not self.megatron_amp_O2) and (self.autocast_dtype in [torch.float16, torch.bfloat16]) else False
         )
 
         if self.pipeline_parallel:
@@ -165,8 +169,10 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         if self.first_stage_of_pipeline() and self.virtual_prompt_style in [
             VirtualPromptStyle.P_TUNING,
         ]:
-
-            self.word_embeddings = self.frozen_model.model.language_model.embedding.word_embeddings
+            if self.frozen_model.mcore_gpt:
+                self.word_embeddings = self.frozen_model.model.embedding.word_embeddings
+            else:
+                self.word_embeddings = self.frozen_model.model.language_model.embedding.word_embeddings
 
         self.padded_vocab_size = self.frozen_model.padded_vocab_size
 
@@ -174,7 +180,10 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         self.pseudo_tokens = get_pseudo_tokens(self.max_virtual_tokens)
         if isinstance(self.tokenizer, SentencePieceTokenizer):
             if not self.tokenizer.legacy:
-                self.tokenizer.pad_token = self.tokenizer.ids_to_tokens([self.tokenizer.pad_id])[0]
+                if self.tokenizer.pad_id != -1:
+                    self.tokenizer.pad_token = self.tokenizer.ids_to_tokens([self.tokenizer.pad_id])[0]
+                else:
+                    self.tokenizer.pad_token = self.tokenizer.ids_to_tokens([self.tokenizer.eos_id])[0]
                 self.tokenizer.bos_token = self.tokenizer.ids_to_tokens([self.tokenizer.bos_id])[0]
                 self.tokenizer.eos_token = self.tokenizer.ids_to_tokens([self.tokenizer.eos_id])[0]
                 self.tokenizer.legacy = True
@@ -217,6 +226,7 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
                 "add_BOS": True,
                 "all_probs": False,
                 "compute_logprob": False,
+                "end_strings": self.cfg.inference.get('end_strings', ["<|endoftext|>"]),
             }
         elif self.cfg.get("report_validation_metric", False) and not hasattr(self.cfg, 'inference'):
             raise ValueError("Must provide inference parameters for reporting validation metric!")
@@ -244,6 +254,7 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         inference=True,
         set_inference_key_value_memory=False,
         inference_max_sequence_len=None,
+        inference_params=None,
     ):
         """
         Special forward method for p-tuning/prompt-tuning pretrained
@@ -253,7 +264,12 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         # Get embeddings for text tokens and insert virtual token embeddings
         if self.first_stage_of_pipeline():
             input_embeds = self.embed_input(input_ids, taskname_ids, use_cached_reps=inference)
-            if hasattr(self.frozen_model.model.language_model.embedding, "position_embeddings"):
+            if self.frozen_model.mcore_gpt and hasattr(self.frozen_model.model.embedding, "position_embeddings"):
+                position_embeddings = self.frozen_model.model.embedding.position_embeddings(position_ids)
+                encoder_input = input_embeds + position_embeddings
+            elif not self.frozen_model.mcore_gpt and hasattr(
+                self.frozen_model.model.language_model.embedding, "position_embeddings"
+            ):
                 position_embeddings = self.frozen_model.model.language_model.embedding.position_embeddings(
                     position_ids
                 )
@@ -267,15 +283,14 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
             encoder_input = None
 
         # Call forward on GPT model with preprocessed embeddings
-        if self.autocast_dtype == torch.float32:
+        if self.frozen_model.mcore_gpt:
             output = self.frozen_model.model(
                 input_ids=None,
                 position_ids=None,
-                encoder_input=encoder_input,
+                decoder_input=encoder_input,
                 attention_mask=attention_mask,
                 labels=labels,
-                set_inference_key_value_memory=set_inference_key_value_memory,
-                inference_max_sequence_len=inference_max_sequence_len,
+                inference_params=inference_params,
             )
         else:
             output = self.frozen_model.model(
@@ -292,13 +307,12 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
 
     def fwd_bwd_step(self, dataloader_iter, batch_idx, forward_only):
         """
-            Dataloader produces a global batch which is turned into an iterator of microbatches.
-            The iterator of microbatches is then piped through the pipeline using Core's fwd/bwd functions.
+        Dataloader produces a global batch which is turned into an iterator of microbatches.
+        The iterator of microbatches is then piped through the pipeline using Core's fwd/bwd functions.
         """
         # Get seq length of batch
-        batch = next(dataloader_iter)
+        batch, _, _ = next(dataloader_iter)
         _, seq_length = batch[0].shape
-        tensor_shape = [seq_length, get_micro_batch_size(), self.hidden_size]
         data_iter = get_iterator_k_split(batch, get_num_microbatches())
 
         fwd_bwd_function = get_forward_backward_func()
@@ -309,11 +323,8 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
             model=[self],
             num_microbatches=get_num_microbatches(),
             forward_only=forward_only,
-            tensor_shape=tensor_shape,
-            dtype=self.autocast_dtype,
-            grad_scaler=self.trainer.precision_plugin.scaler.scale if self.cfg.precision == 16 else None,
-            sequence_parallel=self.cfg.get('sequence_parallel', False),
-            enable_autocast=self.enable_autocast,
+            seq_length=seq_length,
+            micro_batch_size=get_micro_batch_size(),
         )
 
         # only the last stages of the pipeline return losses
@@ -328,10 +339,10 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
 
         return loss_mean
 
-    def training_step(self, dataloader_iter, batch_idx):
+    def training_step(self, dataloader_iter):
         # we zero grads here because we also call backward in the megatron-core fwd/bwd functions
         self._optimizer.zero_grad()
-        batch = next(dataloader_iter)
+        batch, batch_idx, _ = next(dataloader_iter)
         loss_mean = self.fwd_bwd_step(itertools.chain([batch]), batch_idx, forward_only=False)
         self.allreduce_gradients()
 
@@ -340,7 +351,7 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         # we can avoid this broadcast by updating the PTL log function to accept specific ranks
         torch.distributed.broadcast(loss_mean, get_last_rank())
 
-        if self.cfg.precision == 16 and hasattr(self.trainer.precision_plugin.scaler, "_scale"):
+        if self.torch_dtype == torch.float16 and hasattr(self.trainer.precision_plugin.scaler, "_scale"):
             loss_scale = self.trainer.precision_plugin.scaler._scale
             if loss_scale is not None:
                 self.log('loss_scale', loss_scale, batch_size=1)
@@ -352,20 +363,21 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         return loss_mean
 
     def backward(self, *args, **kwargs):
-        """ LightningModule hook to do backward.
-            We want this to do nothing since we run backward in the fwd/bwd functions from megatron-core.
-            No need to call it here.
+        """LightningModule hook to do backward.
+        We want this to do nothing since we run backward in the fwd/bwd functions from megatron-core.
+        No need to call it here.
         """
         return
 
     def optimizer_zero_grad(self, *args, **kwargs):
-        """ LightningModule hook to zero grad.
-            We want this to do nothing as we are zeroing grads during the training_step.
+        """LightningModule hook to zero grad.
+        We want this to do nothing as we are zeroing grads during the training_step.
         """
         return
 
-    def validation_step(self, dataloader_iter, batch_idx):
-        batch = next(dataloader_iter)
+    def validation_step(self, dataloader_iter):
+        mode = 'test' if self.trainer.testing else 'val'
+        batch, batch_idx, _ = next(dataloader_iter)
         gbs = self.cfg.get('validation_global_batch_size', self.cfg.global_batch_size)
         self._reconfigure_and_process_inference_batch(batch[0].size(0), gbs)
         loss_mean = self.fwd_bwd_step(itertools.chain([batch]), batch_idx, forward_only=True)
@@ -403,12 +415,33 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
                 label = self.tokenizer.ids_to_text(label)
                 preds_text.append(pred)
                 labels_text.append(label)
-
+            if mode == 'val':
+                self.validation_step_outputs.append(
+                    {
+                        'loss': loss_mean,
+                        'preds': preds_text,
+                        'labels': labels_text,
+                    }
+                )
+            else:
+                self.test_step_outputs.append(
+                    {
+                        'loss': loss_mean,
+                        'preds': preds_text,
+                        'labels': labels_text,
+                    }
+                )
             return {
                 'loss': loss_mean,
                 'preds': preds_text,
                 'labels': labels_text,
             }
+
+        (
+            self.validation_step_outputs.append({'loss': loss_mean})
+            if mode == 'val'
+            else self.test_step_outputs.append({'loss': loss_mean})
+        )
         return {'loss': loss_mean}
 
     def on_train_epoch_start(self) -> None:
@@ -423,13 +456,13 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         self._reconfigure_batch_sizes(gbs, mbs)
         return super().on_validation_epoch_start()
 
-    def validation_epoch_end(self, outputs):
-        if len(outputs) == 0:
+    def on_validation_epoch_end(self):
+        if not self.validation_step_outputs:
             return
 
         if parallel_state.is_pipeline_last_stage():
             # only the last pipeline parallel stages return loss
-            averaged_loss = torch.stack([i['loss'] for i in outputs]).mean()
+            averaged_loss = torch.stack([i['loss'] for i in self.validation_step_outputs]).mean()
         else:
             averaged_loss = torch.tensor(0.0).cuda()
 
@@ -442,8 +475,8 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         if self.cfg.get("report_validation_metric", False):
             gather_results = [None for _ in range(parallel_state.get_data_parallel_world_size())]
 
-            all_preds = list(itertools.chain(*[item['preds'] for item in outputs]))
-            all_labels = list(itertools.chain(*[item['labels'] for item in outputs]))
+            all_preds = list(itertools.chain(*[item['preds'] for item in self.validation_step_outputs]))
+            all_labels = list(itertools.chain(*[item['labels'] for item in self.validation_step_outputs]))
 
             assert len(all_preds) == len(all_labels)
 
@@ -460,7 +493,8 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
                 gather_results_dedup = list(set(itertools.chain(*gather_results)))
 
                 val_metric_dict = self.validation_metric.get_score(
-                    [i[1] for i in gather_results_dedup], [i[0] for i in gather_results_dedup],
+                    [i[1] for i in gather_results_dedup],
+                    [i[0] for i in gather_results_dedup],
                 )
 
                 for metric, val in val_metric_dict.items():
@@ -476,13 +510,21 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         gbs = self.cfg.global_batch_size
         mbs = self.cfg.micro_batch_size
         self._reconfigure_batch_sizes(gbs, mbs)
+        self.validation_step_outputs.clear()  # free memory
 
-    def test_step(self, dataloader_iter, batch_idx):
-        return self.validation_step(dataloader_iter, batch_idx)
+    def test_step(self, dataloader_iter):
+        return self.validation_step(dataloader_iter)
 
-    def test_epoch_end(self, outputs):
-        averaged_loss = average_losses_across_data_parallel_group(outputs)
+    def on_test_epoch_end(self):
+        averaged_loss = average_losses_across_data_parallel_group(self.test_step_outputs)
         logging.info(f'test_loss: {averaged_loss[0]}')
+        self.test_step_outputs.clear()  # free memory
+
+    def setup(self, stage=None):
+        super().setup(stage)
+
+        if self.cfg.get('transformer_engine', False) or self.cfg.get('mcore_gpt', False):
+            self.frozen_model.setup_transformer_engine_tp_groups()
 
     def setup_training_data(self, training_data_config=None):
         if self.cfg.data.get('train_ds', None):
@@ -609,9 +651,9 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
             drop_last=drop_last,
             num_workers=num_workers,
             pin_memory=pin_memory,
-            persistent_workers=True
-            if num_workers > 0
-            else False,  # (@adithyare and @eharper) We need this to make spawn=True to work.
+            persistent_workers=(
+                True if num_workers > 0 else False
+            ),  # (@adithyare and @eharper) We need this to make spawn=True to work.
         )
 
         return dataset, dataloader
@@ -628,7 +670,7 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
 
     def get_forward_output_and_loss_func(self):
         def fwd_output_and_loss_func(dataloader_iter, model):
-            batch = next(dataloader_iter)
+            batch, _, _ = next(dataloader_iter)
             batch = [x.cuda(non_blocking=True) for x in batch]
             input_ids, labels, loss_mask, position_ids, attention_mask, taskname_ids = batch
             output_tensor = model(input_ids, position_ids, attention_mask, taskname_ids, labels, inference=False)
@@ -651,7 +693,7 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
         """
 
         def fwd_output_only_func(dataloader_iter, model):
-            batch = next(dataloader_iter)
+            batch, _, _ = next(dataloader_iter)
             extra_arg = {}
             (
                 tokens,
@@ -666,11 +708,24 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
             attention_mask = attention_mask.cuda()
             position_ids = position_ids.cuda()
             task_ids = task_ids.cuda()
-            extra_arg['set_inference_key_value_memory'] = set_inference_key_value_memory[0].item()
-            extra_arg['inference_max_sequence_len'] = inference_max_sequence_len[0].item()
+
+            if self.frozen_model.mcore_gpt:
+                # if first step, then clear KV cache, otherwise reuse inference_paarms
+                if set_inference_key_value_memory[0].item():
+                    self.inference_params = InferenceParams(
+                        max_batch_size=tokens.size(0), max_sequence_length=inference_max_sequence_len[0].item()
+                    )
+                extra_arg['inference_params'] = self.inference_params
+            else:
+                extra_arg['set_inference_key_value_memory'] = set_inference_key_value_memory[0].item()
+                extra_arg['inference_max_sequence_len'] = inference_max_sequence_len[0].item()
 
             output_tensor = model(tokens, position_ids, attention_mask, task_ids, **extra_arg)
 
+            # Advance inference sequence offset.
+            if self.inference_params:
+                self.inference_params.sequence_len_offset += output_tensor.size(1)
+
             def id_func(output_tensor):
                 return output_tensor, {'logits': output_tensor}
 
@@ -687,7 +742,7 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
     ):
 
         # check whether the DDP is initialized
-        if parallel_state.is_unitialized():
+        if not parallel_state.is_initialized():
 
             def dummy():
                 return
@@ -753,6 +808,8 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
                 "add_BOS": inference_config["add_BOS"],
                 "all_probs": inference_config["all_probs"],
                 "compute_logprob": inference_config["compute_logprob"],
+                "compute_attention_mask": inference_config.get("compute_attention_mask", True),
+                "end_strings": inference_config.get('end_strings', ["<|endoftext|>"]),
             }
 
             task_ids, processed_inputs = batch
@@ -771,7 +828,7 @@ class MegatronGPTPromptLearningModel(MegatronBasePromptLearningModel):
 def get_pseudo_tokens(num_virtual_tokens):
     """
     Takes in an integer and returns a list of strings where each string
-    is a numbered virtual token placeholder. If 
+    is a numbered virtual token placeholder. If
     num_virtual_tokens = 3, then this function returns:
 
     ["<prompt_0>", "<prompt_1>", "<prompt_2>"]
@@ -779,7 +836,7 @@ def get_pseudo_tokens(num_virtual_tokens):
     Args:
         num_virtual_tokens: (int) Number of virtual token strings you want to make
 
-    returns a list of string. 
+    returns a list of string.
 
     """
     pseudo_tokens = [
