@@ -21,20 +21,23 @@ import itertools
 from typing import Any
 
 import torch
+from lightning.pytorch.trainer.trainer import Trainer
 from omegaconf.dictconfig import DictConfig
 from omegaconf.omegaconf import open_dict
-from pytorch_lightning.trainer.trainer import Trainer
 
 from nemo.collections.common.parts.adapter_modules import LinearAdapterConfig
-from nemo.collections.nlp.models.language_modeling.megatron_finetune_model import MegatronT5FinetuneModel
 from nemo.collections.nlp.models.language_modeling.megatron_t5_model import MegatronT5Model
 from nemo.collections.nlp.models.language_modeling.megatron_t5_prompt_learning_model import (
     MegatronT5PromptLearningModel,
 )
+from nemo.collections.nlp.models.language_modeling.megatron_t5_sft_model import MegatronT5SFTModel
 from nemo.collections.nlp.modules.common import VirtualPromptStyle
 from nemo.collections.nlp.modules.common.megatron.adapters.parallel_adapters import (
     AdapterName,
     InfusedAdapterConfig,
+    LoraKQVAdapterConfig,
+    LoraKVAdapterConfig,
+    LoraQAdapterConfig,
     MLPInfusedAdapterConfig,
     ParallelLinearAdapterConfig,
 )
@@ -57,7 +60,15 @@ class MegatronT5BaseAdapterModel(MegatronT5PromptLearningModel):
         self.adapter_name_keys = []
 
     def forward(
-        self, input_ids, dec_input, enc_mask, dec_mask, position_ids, taskname_ids, labels=None, inference=False,
+        self,
+        input_ids,
+        dec_input,
+        enc_mask,
+        dec_mask,
+        position_ids,
+        taskname_ids,
+        labels=None,
+        inference=False,
     ):
         # Call forward on T5 model with preprocessed embeddings
         if self.autocast_dtype == torch.float32:
@@ -143,8 +154,8 @@ class MegatronT5BaseAdapterModel(MegatronT5PromptLearningModel):
             'enc_inputs': processed_inputs,
         }
 
-    def validation_step(self, dataloader_iter, batch_idx, inference=False):
-        batch = next(dataloader_iter)
+    def validation_step(self, dataloader_iter):
+        batch, batch_idx, _ = next(dataloader_iter)
         enc_input, dec_input, labels, loss_mask, enc_mask, dec_mask, position_ids, taskname_ids = batch
 
         mode = self.training
@@ -159,6 +170,7 @@ class MegatronT5BaseAdapterModel(MegatronT5PromptLearningModel):
         else:
             metrics = {'loss': loss_mean}
 
+        self.validation_step_outputs.append(metrics)
         self.train(mode=mode)
         return metrics
 
@@ -175,11 +187,11 @@ class MegatronT5BaseAdapterModel(MegatronT5PromptLearningModel):
         )
 
         # Special ids to text function to handle stripping <eos> and special tokens with sentencepiece tokenizers.
-        preds_text = MegatronT5FinetuneModel.ids_to_text(predicted_token_ids, self.tokenizer)
-        input_text = MegatronT5FinetuneModel.ids_to_text(enc_input, self.tokenizer)
+        preds_text = MegatronT5SFTModel.ids_to_text(predicted_token_ids, self.tokenizer)
+        input_text = MegatronT5SFTModel.ids_to_text(enc_input, self.tokenizer)
 
         if labels is not None:
-            labels_text = MegatronT5FinetuneModel.ids_to_text(labels, self.tokenizer)
+            labels_text = MegatronT5SFTModel.ids_to_text(labels, self.tokenizer)
         else:
             labels_text = [None] * len(preds_text)
 
@@ -191,13 +203,13 @@ class MegatronT5BaseAdapterModel(MegatronT5PromptLearningModel):
 
     def setup_optimizer_param_groups(self):
         """
-        ModelPT override. Optimizer will get self._optimizer_param_groups. 
+        ModelPT override. Optimizer will get self._optimizer_param_groups.
         Makes two optimizer param groups, one for the frozen model params
-        and one for the prompt-table/prompt-encoder params. The learning 
+        and one for the prompt-table/prompt-encoder params. The learning
         rate for the frozen model's params will always be zero effectively
         freezing the model's params but still allowing for the needed gradients
-        to be passed around in pipeline parallel models. The prompt-encoder 
-        and/or prompt table will use the learning rate set by the user. 
+        to be passed around in pipeline parallel models. The prompt-encoder
+        and/or prompt table will use the learning rate set by the user.
         """
         self.frozen_model.freeze()  # Freeze the entire model
         opt_params = []
@@ -262,7 +274,7 @@ class MegatronT5BaseAdapterModel(MegatronT5PromptLearningModel):
 
     def load_state_dict(self, state_dict, strict: bool = True):
         """
-        Loads a state_dict expecting the state_dict to contain key,values 
+        Loads a state_dict expecting the state_dict to contain key,values
         only for the adapter parameters.
         """
         for name, module in self.frozen_model.named_modules():
@@ -274,11 +286,11 @@ class MegatronT5BaseAdapterModel(MegatronT5PromptLearningModel):
                         adapter_module.load_state_dict(state_dict[state_adapter_key], strict)
                 module.set_enabled_adapters(enabled=True)
 
-    def validation_epoch_end(self, outputs):
+    def on_validation_epoch_end(self):
         if self.cfg.get('pipeline_model_parallel_size', 1) > 1:
             if parallel_state.is_pipeline_last_stage():
                 # only the last pipeline parallel stages return loss
-                averaged_loss = torch.stack([i['loss'] for i in outputs]).mean()
+                averaged_loss = torch.stack([i['loss'] for i in self.validation_step_outputs]).mean()
             else:
                 averaged_loss = torch.tensor(0.0).cuda()
 
@@ -289,15 +301,15 @@ class MegatronT5BaseAdapterModel(MegatronT5PromptLearningModel):
             logging.info(f'Validation loss: {averaged_loss}')
 
         else:
-            averaged_loss = torch.stack([item['loss'] for item in outputs]).mean()
+            averaged_loss = torch.stack([item['loss'] for item in self.validation_step_outputs]).mean()
             logging.info(f'Validation loss: {averaged_loss}')
             self.log('val_loss', averaged_loss, prog_bar=True, rank_zero_only=True, batch_size=1)
 
         if self.cfg.get('report_validation_accuracy', False):
             gather_results = [None for _ in range(parallel_state.get_data_parallel_world_size())]
-            all_preds = list(itertools.chain(*[item['predicted_token_ids'] for item in outputs]))
-            all_labels = list(itertools.chain(*[item['labels'] for item in outputs]))
-            all_inputs = list(itertools.chain(*[item['enc_inputs'] for item in outputs]))
+            all_preds = list(itertools.chain(*[item['predicted_token_ids'] for item in self.validation_step_outputs]))
+            all_labels = list(itertools.chain(*[item['labels'] for item in self.validation_step_outputs]))
+            all_inputs = list(itertools.chain(*[item['enc_inputs'] for item in self.validation_step_outputs]))
 
             assert len(all_preds) == len(all_labels)
             assert len(all_preds) == len(all_inputs)
@@ -315,7 +327,7 @@ class MegatronT5BaseAdapterModel(MegatronT5PromptLearningModel):
                 gather_results_dedup = list(set(itertools.chain(*gather_results)))
 
                 correct = 0
-                for (input, pred, label) in gather_results_dedup:
+                for input, pred, label in gather_results_dedup:
                     if pred == label:
                         correct += 1
 
@@ -331,6 +343,7 @@ class MegatronT5BaseAdapterModel(MegatronT5PromptLearningModel):
         gbs = self.cfg.global_batch_size
         mbs = self.cfg.micro_batch_size
         self._reconfigure_batch_sizes(gbs, mbs)
+        self.validation_step_outputs.clear()  # free memory
 
 
 class MegatronT5AdapterLearningModel(MegatronT5BaseAdapterModel):
@@ -420,6 +433,136 @@ class MegatronT5AdapterLearningModel(MegatronT5BaseAdapterModel):
         pass
 
 
+class MegatronT5LoraModel(MegatronT5BaseAdapterModel):
+    """
+    TODO  (@adithyare)
+    """
+
+    def __init__(self, cfg: DictConfig, trainer: Trainer):
+        super().__init__(cfg, trainer)
+        # assert cfg.lora_tuning.get('adapter_dim', 0) > 0, "adapter_dim has not been set."
+        # assert (
+        #     cfg.lora_tuning.adapter_dim % cfg.tensor_model_parallel_size == 0
+        # ), "The adapter dim should be divisible by tensor_model_parallel_size."
+
+        encoder_adapter_name_keys = [AdapterName.LORA_KQV_ADAPTER]
+        decoder_adapter_name_keys = [
+            AdapterName.LORA_KQV_ADAPTER,
+            AdapterName.LORA_KV_ADAPTER,
+            AdapterName.LORA_Q_ADAPTER,
+        ]
+
+        # add adapter keys to the list -> to update state dict
+        self.adapter_name_keys = encoder_adapter_name_keys + decoder_adapter_name_keys
+
+        frozen_model_cfg = MegatronT5Model.restore_from(
+            cfg.get('language_model_path'), trainer=trainer, return_config=True
+        )
+        for _, layer in self.frozen_model.named_modules():
+            if hasattr(layer, 'activations_checkpoint_method'):
+                layer.activations_checkpoint_method = (
+                    None  # (@adithyare) adapter learning does not support activations checkpointing atm.
+                )
+
+        self.frozen_model.freeze()
+        logging.info(f'Before adding adapters:\n{self.frozen_model.summarize()}')
+        if self.megatron_amp_O2:
+            encoder = self.frozen_model.enc_dec_model.module.enc_dec_model.encoder
+            decoder = self.frozen_model.enc_dec_model.module.enc_dec_model.decoder
+        else:
+            encoder = self.frozen_model.enc_dec_model.enc_dec_model.encoder
+            decoder = self.frozen_model.enc_dec_model.enc_dec_model.decoder
+
+        if encoder:
+            encoder_cfg = self._get_component_cfg('encoder', frozen_model_cfg, cfg)
+            self._add_adapters_to_component(encoder, encoder_cfg, encoder_adapter_name_keys)
+            logging.info(f'Adding encoder adapters:\n{self.frozen_model.summarize()}')
+
+        if decoder:
+            decoder_cfg = self._get_component_cfg('decoder', frozen_model_cfg, cfg)
+            self._add_adapters_to_component(decoder, decoder_cfg, decoder_adapter_name_keys)
+            logging.info(f'Adding decoder adapters:\n{self.frozen_model.summarize()}')
+
+    def _add_adapters_to_component(self, component, component_cfg, adapter_name_keys):
+        for _, module in component.named_modules():
+            if isinstance(module, adapter_mixins.AdapterModuleMixin):
+                for adapter_key in adapter_name_keys:
+                    adapter_cfg = self._get_adapter_cfg(component_cfg, adapter_key)
+                    if model_utils.import_class_by_path(adapter_cfg._target_) in module.get_accepted_adapter_types():
+                        module.add_adapter(name=adapter_key, cfg=adapter_cfg)
+                        print(f"in adding {adapter_key}")
+
+    def _get_component_cfg(self, component_name, frozen_model_cfg, cfg):
+        if component_name in frozen_model_cfg:
+            component_cfg = frozen_model_cfg.get(component_name)
+            with open_dict(component_cfg):
+                component_cfg.tensor_model_parallel_size = frozen_model_cfg.tensor_model_parallel_size
+                component_cfg.lora_tuning = cfg.lora_tuning
+        else:
+            component_cfg = frozen_model_cfg
+            with open_dict(component_cfg):
+                component_cfg.lora_tuning = cfg.lora_tuning
+        return component_cfg
+
+    def _get_adapter_cfg(self, component_cfg, adapter_key):
+        if component_cfg.kv_channels is None:
+            assert (
+                component_cfg.hidden_size % component_cfg.num_attention_heads == 0
+            ), 'hidden_size must be divisible by num_attention_heads if kv_channels is None'
+            kv_channels = component_cfg.hidden_size // component_cfg.num_attention_heads
+        else:
+            kv_channels = component_cfg.kv_channels
+        projection_size = kv_channels * component_cfg.num_attention_heads
+
+        if adapter_key == AdapterName.LORA_KQV_ADAPTER:
+            adapter_cfg = LoraKQVAdapterConfig(
+                in_features=component_cfg.hidden_size,
+                out_features=3 * projection_size,
+                dim=component_cfg.lora_tuning.kqv_adapter_dim,
+                norm_position="none",
+                norm_type="none",
+                activation="identity",
+                column_init_method=component_cfg.lora_tuning.get("column_init_method", "normal"),
+                row_init_method=component_cfg.lora_tuning.get("row_init_method", "zero"),
+                gather_output=False,
+                dropout=0.0,
+            )
+        elif adapter_key == AdapterName.LORA_KV_ADAPTER:
+            adapter_cfg = LoraKVAdapterConfig(
+                in_features=component_cfg.hidden_size,
+                out_features=2 * projection_size,
+                dim=component_cfg.lora_tuning.kv_adapter_dim,
+                norm_position="none",
+                norm_type="none",
+                activation="identity",
+                column_init_method=component_cfg.lora_tuning.get("column_init_method", "normal"),
+                row_init_method=component_cfg.lora_tuning.get("row_init_method", "zero"),
+                gather_output=False,
+                dropout=0.0,
+            )
+        elif adapter_key == AdapterName.LORA_Q_ADAPTER:
+            adapter_cfg = LoraQAdapterConfig(
+                in_features=component_cfg.hidden_size,
+                out_features=1 * projection_size,
+                dim=component_cfg.lora_tuning.q_adapter_dim,
+                norm_position="none",
+                norm_type="none",
+                activation="identity",
+                column_init_method=component_cfg.lora_tuning.get("column_init_method", "normal"),
+                row_init_method=component_cfg.lora_tuning.get("row_init_method", "zero"),
+                gather_output=False,
+                dropout=0.0,
+            )
+        else:
+            raise RuntimeError("Unexpected adapter key name..")
+
+        return adapter_cfg
+
+    @classmethod
+    def list_available_models(cls):
+        pass
+
+
 class MegatronT5InfusedAdapterModel(MegatronT5BaseAdapterModel):
     """
     MegatronGPTInfusedAdapterModel is a model that combines a base model (GPTModel) with a "Infused Adapter that can Inhibiting and Amplify Inner Activations", known as IA3.
@@ -428,8 +571,8 @@ class MegatronT5InfusedAdapterModel(MegatronT5BaseAdapterModel):
     Three adapter's are inserted into each Transformer layer in the base GPT Model. Each adapter is basically a vector that simply scales the key, value or ffn hidden representations.
 
     It is assumed that these set of adapters will then be trained for a specific task.
-    Once trained, the adapter weights will be saved and can be re-loaded 
-    and infused into the same GPT Model for inference. 
+    Once trained, the adapter weights will be saved and can be re-loaded
+    and infused into the same GPT Model for inference.
     """
 
     def __init__(self, cfg: DictConfig, trainer: Trainer):
@@ -539,7 +682,7 @@ class MegatronT5InfusedAdapterModel(MegatronT5BaseAdapterModel):
 
     def load_state_dict(self, state_dict, strict: bool = True):
         """
-        Loads a state_dict expecting the state_dict to contain key,values 
+        Loads a state_dict expecting the state_dict to contain key,values
         only for the adapter parameters.
         """
         encoder = self.frozen_model.enc_dec_model.enc_dec_model.encoder
