@@ -17,8 +17,8 @@ from typing import List, Optional
 
 import numpy as np
 import torch
+from lightning.pytorch import Trainer
 from omegaconf import DictConfig
-from pytorch_lightning import Trainer
 from transformers.models.bert.tokenization_bert import BasicTokenizer
 
 from nemo.collections.common.losses import SpanningLoss
@@ -31,12 +31,15 @@ from nemo.collections.nlp.modules.common import TokenClassifier
 from nemo.collections.nlp.parts.utils_funcs import tensor2list
 from nemo.core.classes.common import PretrainedModelInfo, typecheck
 from nemo.utils import logging
+from nemo.utils.decorators import deprecated_warning
 
 
 class BERTQAModel(BaseQAModel):
-    """ BERT model with a QA (token classification) head """
+    """BERT model with a QA (token classification) head"""
 
     def __init__(self, cfg: DictConfig, trainer: Trainer = None):
+        # deprecation warning
+        deprecated_warning("BERTQAModel")
 
         super().__init__(cfg=cfg, trainer=trainer, no_lm_init=False)
         self.classifier = TokenClassifier(
@@ -76,19 +79,34 @@ class BERTQAModel(BaseQAModel):
             'start_logits': start_logits,
             'end_logits': end_logits,
         }
-        return {f'{prefix}_loss': loss, f'{prefix}_tensors': tensors}
+        loss = {f'{prefix}_loss': loss, f'{prefix}_tensors': tensors}
+        if prefix == "val":
+            self.validation_step_outputs.append(loss)
+        else:
+            self.test_step_outputs.append(loss)
+
+        return loss
 
     def test_step(self, batch, batch_idx):
         return self.validation_step(batch, batch_idx)
 
-    def validation_epoch_end(self, outputs):
+    def on_validation_epoch_end(self):
         prefix = "test" if self.trainer.testing else "val"
 
-        avg_loss = torch.stack([x[f'{prefix}_loss'] for x in outputs]).mean()
+        if prefix == 'val':
+            avg_loss = torch.stack([x[f'{prefix}_loss'] for x in self.validation_step_outputs]).mean()
 
-        unique_ids = torch.cat([x[f'{prefix}_tensors']['unique_ids'] for x in outputs])
-        start_logits = torch.cat([x[f'{prefix}_tensors']['start_logits'] for x in outputs])
-        end_logits = torch.cat([x[f'{prefix}_tensors']['end_logits'] for x in outputs])
+            unique_ids = torch.cat([x[f'{prefix}_tensors']['unique_ids'] for x in self.validation_step_outputs])
+            start_logits = torch.cat([x[f'{prefix}_tensors']['start_logits'] for x in self.validation_step_outputs])
+            end_logits = torch.cat([x[f'{prefix}_tensors']['end_logits'] for x in self.validation_step_outputs])
+            self.validation_step_outputs.clear()  # free memory
+        else:
+            avg_loss = torch.stack([x[f'{prefix}_loss'] for x in self.test_step_outputs]).mean()
+
+            unique_ids = torch.cat([x[f'{prefix}_tensors']['unique_ids'] for x in self.test_step_outputs])
+            start_logits = torch.cat([x[f'{prefix}_tensors']['start_logits'] for x in self.test_step_outputs])
+            end_logits = torch.cat([x[f'{prefix}_tensors']['end_logits'] for x in self.test_step_outputs])
+            self.test_step_outputs.clear()  # free memory
 
         all_unique_ids = []
         all_start_logits = []
@@ -140,8 +158,8 @@ class BERTQAModel(BaseQAModel):
             logging.info(f"{prefix} {eval_key}: {eval_results[eval_key]}")
             self.log(f"{prefix}_{eval_key}", eval_results[eval_key])
 
-    def test_epoch_end(self, outputs):
-        return self.validation_epoch_end(outputs)
+    def on_test_epoch_end(self):
+        return self.on_validation_epoch_end()
 
     @typecheck()
     def forward(self, input_ids, attention_mask, token_type_ids):
@@ -175,7 +193,7 @@ class BERTQAModel(BaseQAModel):
             num_samples: number of samples to use of inference data. Default: -1 if all data should be used.
             output_nbest_file: optional output file for writing out nbest list
             output_prediction_file: optional output file for writing out predictions
-            
+
         Returns:
             model predictions, model nbest list
         """
@@ -194,7 +212,10 @@ class BERTQAModel(BaseQAModel):
             logging.set_verbosity(logging.WARNING)
 
             infer_datalayer = self.setup_inference_data(
-                file, batch_size=batch_size, num_samples=num_samples, num_workers=2,
+                file,
+                batch_size=batch_size,
+                num_samples=num_samples,
+                num_workers=2,
             )
 
             all_logits = []
@@ -229,7 +250,9 @@ class BERTQAModel(BaseQAModel):
 
             if output_prediction_file:
                 QAMetrics.dump_predicted_answers_to_file(
-                    output_prediction_file, infer_datalayer.dataset.examples, all_predictions,
+                    output_prediction_file,
+                    infer_datalayer.dataset.examples,
+                    all_predictions,
                 )
 
             if output_nbest_file:
@@ -309,7 +332,7 @@ class BERTQAModel(BaseQAModel):
         all_predictions = collections.OrderedDict()
         all_nbest_json = collections.OrderedDict()
         scores_diff_json = collections.OrderedDict()
-        for (example_index, example) in enumerate(examples):
+        for example_index, example in enumerate(examples):
 
             # finish this loop if we went through all batch examples
             if example_index >= len(unique_ids):
@@ -334,7 +357,7 @@ class BERTQAModel(BaseQAModel):
             null_start_logit = 0
             # end logit at the slice with min null score
             null_end_logit = 0
-            for (feature_index, feature) in enumerate(curr_features):
+            for feature_index, feature in enumerate(curr_features):
                 pos = unique_id_to_pos[feature.unique_id]
                 start_indexes = self._get_best_indexes(start_logits[pos], n_best_size)
                 end_indexes = self._get_best_indexes(end_logits[pos], n_best_size)
@@ -453,7 +476,7 @@ class BERTQAModel(BaseQAModel):
             probs = _compute_softmax(total_scores)
 
             nbest_json = []
-            for (i, entry) in enumerate(nbest):
+            for i, entry in enumerate(nbest):
                 output = collections.OrderedDict()
                 output["question"] = example.question_text
                 output["text"] = entry.text
@@ -516,7 +539,7 @@ class BERTQAModel(BaseQAModel):
         return data_loader
 
     def _get_best_indexes(self, logits, n_best_size):
-        """ Get the n-best logits from a list """
+        """Get the n-best logits from a list"""
 
         best_indices = np.argsort(logits)[::-1]
 
@@ -555,7 +578,7 @@ class BERTQAModel(BaseQAModel):
         def _strip_spaces(text):
             ns_chars = []
             ns_to_s_map = collections.OrderedDict()
-            for (i, c) in enumerate(text):
+            for i, c in enumerate(text):
                 if c == " ":
                     continue
                 ns_to_s_map[len(ns_chars)] = i
@@ -584,14 +607,16 @@ class BERTQAModel(BaseQAModel):
         if len(orig_ns_text) != len(tok_ns_text):
             if verbose_logging:
                 logging.warning(
-                    "Length not equal after stripping spaces: '%s' vs '%s'", orig_ns_text, tok_ns_text,
+                    "Length not equal after stripping spaces: '%s' vs '%s'",
+                    orig_ns_text,
+                    tok_ns_text,
                 )
             return orig_text
 
         # We then project the characters in `pred_text` back to `orig_text` using
         # the character-to-character alignment.
         tok_s_to_ns_map = {}
-        for (i, tok_index) in tok_ns_to_s_map.items():
+        for i, tok_index in tok_ns_to_s_map.items():
             tok_s_to_ns_map[tok_index] = i
 
         orig_start_position = None
