@@ -18,7 +18,7 @@ import torch
 from torch import nn as nn
 
 from nemo.collections.common.parts import transformer_weights_init
-from nemo.core.classes import Exportable, NeuralModule, typecheck
+from nemo.core.classes import Exportable, NeuralModule
 from nemo.core.neural_types import ChannelType, NeuralType
 
 __all__ = ['Classifier']
