@@ -12,7 +12,4 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from nemo.collections.nlp.modules.common.megatron.megatron_utils import (
-    get_megatron_checkpoint,
-    get_megatron_lm_models_list,
-)
+from .megatron_utils import get_megatron_checkpoint, get_megatron_lm_models_list
