@@ -14,7 +14,9 @@
 
 """Utilities for generating text."""
 
+import os
 import pickle
+import re
 from collections.abc import Iterable
 from functools import partial
 from typing import Callable, Tuple
@@ -22,21 +24,20 @@ from typing import Callable, Tuple
 import numpy as np
 import torch
 import torch.nn.functional as F
+from lightning.fabric.utilities.seed import seed_everything
 
 from nemo.collections.common.tokenizers.tabular_tokenizer import TabularTokenizer
+from nemo.collections.multimodal.data.neva.conversation import (
+    DEFAULT_IM_END_TOKEN,
+    DEFAULT_IM_START_TOKEN,
+    DEFAULT_IMAGE_PATCH_TOKEN,
+    DEFAULT_VID_END_TOKEN,
+    DEFAULT_VID_START_TOKEN,
+)
 from nemo.collections.nlp.modules.common.megatron.utils import get_ltor_masks_and_position_ids
 from nemo.collections.nlp.modules.common.text_generation_strategy import model_inference_strategy_dispatcher
 from nemo.collections.nlp.modules.common.transformer.text_generation import LengthParam, OutputType, SamplingParam
-from nemo.utils import AppState
-
-try:
-    from apex.transformer.pipeline_parallel.utils import _reconfigure_microbatch_calculator
-
-    HAVE_APEX = True
-
-except (ImportError, ModuleNotFoundError):
-
-    HAVE_APEX = False
+from nemo.utils import AppState, logging
 
 try:
     from megatron.core import parallel_state, tensor_parallel
@@ -47,10 +48,20 @@ except (ImportError, ModuleNotFoundError):
 
     HAVE_MEGATRON_CORE = False
 
+try:
+    from megatron.core.num_microbatches_calculator import reconfigure_num_microbatches_calculator
+
+except (ImportError, ModuleNotFoundError):
+    logging.warning("Megatron num_microbatches_calculator not found, using Apex version.")
+    from apex.transformer.pipeline_parallel.utils import (
+        _reconfigure_microbatch_calculator as reconfigure_num_microbatches_calculator,
+    )
+
 __all__ = [
     "get_default_sampling_params",
     "get_default_length_params",
     "megatron_gpt_generate",
+    "megatron_neva_generate",
     "get_computeprob_response",
     "generate",
     "sample_token_greedy",
@@ -69,6 +80,7 @@ def get_default_sampling_params():
         "add_BOS": True,
         "all_probs": False,
         "compute_logprob": False,
+        "end_strings": ["<|endoftext|>", "<extra_id_1>"],
     }
 
     return sampling_params
@@ -97,41 +109,212 @@ def megatron_gpt_generate(model, inputs, tokenizer, length_params, sampling_para
             inputs=inputs,
             tokens_to_generate=length_params['max_length'],
             all_probs=sampling_params['all_probs'],
+            compute_logprob=sampling_params['compute_logprob'],
             temperature=sampling_params['temperature'],
             add_BOS=sampling_params['add_BOS'],
             top_k=sampling_params['top_k'],
             top_p=sampling_params['top_p'],
             greedy=sampling_params['use_greedy'],
             repetition_penalty=sampling_params['repetition_penalty'],
+            end_strings=sampling_params['end_strings'],
             min_tokens_to_generate=length_params['min_length'],
+            compute_attention_mask=sampling_params.get("compute_attention_mask", True),
             **strategy_args,
         )
         compute_prob_response = get_computeprob_response(tokenizer, response, inputs)
         return compute_prob_response
 
-    if isinstance(inputs, (list, tuple)):
-        if isinstance(inputs[0], (str, torch.Tensor)):
-            output = generate(
-                model,
-                inputs=inputs,
-                tokens_to_generate=length_params['max_length'],
-                all_probs=sampling_params['all_probs'],
-                temperature=sampling_params['temperature'],
-                add_BOS=sampling_params['add_BOS'],
-                top_k=sampling_params['top_k'],
-                top_p=sampling_params['top_p'],
-                greedy=sampling_params['use_greedy'],
-                repetition_penalty=sampling_params['repetition_penalty'],
-                min_tokens_to_generate=length_params['min_length'],
-                **strategy_args,
+    if not isinstance(inputs, (list, tuple)):
+        raise NotImplementedError(f"unknown type {type(inputs)} is not implemented")
+
+    output = generate(
+        model,
+        inputs=inputs,
+        tokens_to_generate=length_params['max_length'],
+        all_probs=sampling_params['all_probs'],
+        compute_logprob=sampling_params['compute_logprob'],
+        temperature=sampling_params['temperature'],
+        add_BOS=sampling_params['add_BOS'],
+        top_k=sampling_params['top_k'],
+        top_p=sampling_params['top_p'],
+        greedy=sampling_params['use_greedy'],
+        repetition_penalty=sampling_params['repetition_penalty'],
+        end_strings=sampling_params['end_strings'],
+        min_tokens_to_generate=length_params['min_length'],
+        **strategy_args,
+    )
+    return output
+
+
+def decode_time_tokens(tokenizer, text: str, duration: float, time_tokens: list[str], time_token_ids: list[int]):
+    """Decode the time tokens <t0>....<t99> in the text to the actual time in seconds.
+       TO DO: to do time decoding on output ids instead of text
+
+    Args:
+        text (str): _description_
+        duration (float): the total length of the video in seconds
+        time_tokens (list[str]): list of time tokens [<t1>, <t2>, <t3>, ..]
+        time_token_ids (list[str]): list of time token ids [32004, 32005, ....]
+    """
+    output_ids = tokenizer.text_to_ids(text)
+    num_time_tokens = len(time_token_ids)
+    # the original code is len(output_ids) - 1
+    indices = [j for j in range(len(output_ids)) if output_ids[j] in time_token_ids]
+    last_processed = -1
+    new_output_ids = []
+    for j in range(len(indices)):
+        pred_seq = [int(output_ids[k]) for k in range(last_processed + 1, indices[j])]
+        new_output_ids.extend(pred_seq)
+        max_offset = num_time_tokens - 1
+        time_token = tokenizer.ids_to_tokens([output_ids[indices[j]]])[0]
+        time_idx = time_tokens.index(time_token)
+        time = float(time_idx) * duration / max_offset
+        time = min(max(time, 0), duration)
+        time = round(time, 2)
+        # time_str = '<' + str(time) + '>'
+        time_str = '<%s>' % str(time)
+        new_output_ids.extend(tokenizer.text_to_ids(time_str))
+
+        last_processed = indices[j]
+    pred_seq = [int(x) for x in output_ids[last_processed + 1 :]]
+    new_output_ids.extend(pred_seq)
+    output_ids = new_output_ids
+    decoded_text = tokenizer.ids_to_text(output_ids)
+    return decoded_text
+
+
+def encode_time_str(text: str, duration: float, num_time_tokens: int = 100, time_token_template: str = "<t{t}>"):
+    """
+    Encode the common time expression to its time token expression
+    """
+
+    def time_to_string(time):
+        # time is normalized in [0, 1]
+        max_offset = float(num_time_tokens - 1)
+        time = int(np.round(max_offset * time))
+        return time_token_template.format(t=time)
+
+    def repl(match):
+        value = float(match.group(1)) / duration
+        return time_to_string(value) + f"<!|t{value}t|!>"
+
+    text = re.sub(r"<([\d.]{1,20})s>", repl, text)
+    text = re.sub(r"\s([\d.]{1,20})s[\s|\.|,|>]", repl, text)
+    text = re.sub(r"\s([\d.]{1,20}) seconds", repl, text)
+    text = re.sub(r"\s([\d.]{1,20}) second", repl, text)
+
+    # This is to remove the timestamps from the text
+    text = re.sub(r"<!\|t([\d.]+)t\|!>", "", text)
+    return text.strip()
+
+
+def megatron_neva_generate(model, prompt_dict_list, length_params, sampling_params, inference_config, **strategy_args):
+    use_lita = model.cfg.mm_cfg.get('use_lita', False)
+    if use_lita:
+        num_time_tokens = model.cfg.data.get('num_time_tokens', 100)
+        TIME_TOKEN_TEMPLATE = "<t{t}>"
+        time_tokens = [TIME_TOKEN_TEMPLATE.format(t=i) for i in range(num_time_tokens)]
+        time_token_ids = model.tokenizer.tokens_to_ids(time_tokens)
+
+    model_type = model.cfg.mm_cfg.llm.get("model_type", "nvgpt")
+    conv_template = model.cfg.data.get("conv_template", "nvgpt")
+    final_response = []
+    for idx, prompt_dict in enumerate(prompt_dict_list):
+        # determine the media type in the prompt_dict
+        media_type_token = inference_config.inference.get("media_type", "image")
+        if use_lita:
+            if prompt_dict.get("duration") is not None:
+                duration = prompt_dict.get("duration")
+                prompt_dict['prompt'] = encode_time_str(
+                    prompt_dict['prompt'], duration, num_time_tokens, TIME_TOKEN_TEMPLATE
+                )
+            else:
+                print("duration field is not in prompt file, skipping time encoding.")
+        response = generate(
+            model,
+            inputs=prompt_dict.get('prompt'),
+            tokens_to_generate=length_params['max_length'],
+            all_probs=sampling_params['all_probs'],
+            compute_logprob=sampling_params['compute_logprob'],
+            temperature=sampling_params['temperature'],
+            add_BOS=sampling_params['add_BOS'],
+            top_k=sampling_params['top_k'],
+            top_p=sampling_params['top_p'],
+            greedy=sampling_params['use_greedy'],
+            repetition_penalty=sampling_params['repetition_penalty'],
+            end_strings=sampling_params['end_strings'],
+            min_tokens_to_generate=length_params['min_length'],
+            compute_attention_mask=sampling_params.get("compute_attention_mask", True),
+            image_list=prompt_dict.get(media_type_token),
+            **strategy_args,
+        )
+
+        # Middle stages of PP will return None
+        if response is None:
+            continue
+
+        # Regular expression pattern to match the sequence
+        pattern = re.compile(
+            rf'{DEFAULT_IM_START_TOKEN[model_type]}( ⁇ )+{DEFAULT_IM_END_TOKEN[model_type]}'.replace(r'|', r'\|')
+        )
+        pattern_nvgpt = re.compile(
+            rf'{DEFAULT_IM_START_TOKEN[model_type]}({DEFAULT_IMAGE_PATCH_TOKEN[model_type]})+{DEFAULT_IM_END_TOKEN[model_type]}'.replace(
+                r'|', r'\|'
             )
-            return output
-        elif isinstance(inputs[0], dict):
-            raise NotImplementedError("json object not implemented")
+        )
+
+        if use_lita:
+            pattern_lita = re.compile(rf'{DEFAULT_IM_START_TOKEN[model_type]}(.)+{DEFAULT_IM_END_TOKEN[model_type]}')
+            combined_pattern = re.compile(f'{pattern_lita.pattern}')
         else:
-            raise NotImplementedError("unknown type is not implemented")
-    else:
-        raise NotImplementedError("unknown type is not implemented")
+            combined_pattern = re.compile(f'{pattern.pattern}|{pattern_nvgpt.pattern}')
+        clean_text = re.sub(combined_pattern, f"<{media_type_token}>", response['sentences'][0])
+
+        clean_response = clean_text
+
+        if conv_template in ["nvgpt", "nv_steerlm"]:
+            labels_str_regexp = re.compile(f"<extra_id_2>quality:.*\n")
+            last_match_end_position = None
+            for match in re.finditer(labels_str_regexp, clean_response):
+                last_match_end_position = match.end()
+            if last_match_end_position is not None:
+                clean_response = clean_response[last_match_end_position:]
+            clean_response = clean_response.strip("<extra_id_1>")
+        elif conv_template == 'nv_dpo':
+            clean_response = clean_response.split("<extra_id_1>Assistant\n")[-1]
+            clean_response = clean_response.strip("<extra_id_1>")
+        elif conv_template == "llama_2":
+            clean_response = clean_response.rsplit("[/INST] ", 1)[-1]
+        elif conv_template == "llama_3":
+            clean_response = clean_response.rsplit("assistant<|end_header_id|>\n\n", 1)[-1]
+            clean_response = re.sub(r"(<\|eot_id\|>)+$", "", clean_response)
+        elif conv_template == "yi_34b":
+            clean_response = clean_response.split("<|im_start|>assistant\n")[-1]
+            clean_response = clean_response.strip("<|im_end|>")
+        elif conv_template == "v1":
+            clean_response = clean_response.rsplit("ASSISTANT: ", 1)[-1]
+
+        if use_lita:
+            if prompt_dict.get("duration", None) is not None:
+                duration = prompt_dict.get("duration")
+                clean_response = decode_time_tokens(
+                    model.tokenizer, clean_response, duration, time_tokens, time_token_ids
+                )
+            else:
+                print("duration field is not in prompt file, skipping time decoding.")
+        clean_response = clean_response.strip()
+        response["clean_text"] = clean_text
+        response["clean_response"] = clean_response
+        final_response.append(response)
+
+        if torch.cuda.current_device() == 0:
+            print(f"------------- PROMPT {idx} of {len(prompt_dict_list)} ------------ ")
+            print(clean_text)
+            print()
+            print(f"CLEAN RESPONSE: {clean_response}")
+            print("---------------------------------------------\n")
+
+    return final_response
 
 
 def get_computeprob_response(tokenizer, response, inputs):
@@ -154,6 +337,14 @@ def get_computeprob_response(tokenizer, response, inputs):
                     token_len = int(inputs[1][batch_id].item())
                     new_token_id = inputs[0][batch_id][:token_len].tolist()
                     new_text = tokenizer.ids_to_text(new_token_id)
+                else:
+                    raise TypeError(
+                        f"Unsupported type of `inputs[0]`: {type(inputs[0])}. Supported types: `str`, `torch.Tensor`."
+                    )
+            else:
+                raise TypeError(
+                    f"Unsupported type of parameter `inputs`: {type(inputs)}. Supported types: `list` and `tuple`"
+                )
             new_token_ids.append(new_token_id)
             new_tokens.append(response['tokens'][batch_id][:token_len])
             new_texts.append(new_text)
@@ -196,17 +387,17 @@ def tab_logits(logits, min_id, max_id, filter_value=-float('Inf')):
 
 def top_k_logits(logits, top_k=0, top_p=0.0, filter_value=-float('Inf'), started=None):
     """
-       This function has been mostly taken from huggingface conversational
-         ai code at
-         https://medium.com/huggingface/how-to-build-a-state-of-the-art-
-              conversational-ai-with-transfer-learning-2d818ac26313 
+    This function has been mostly taken from huggingface conversational
+      ai code at
+      https://medium.com/huggingface/how-to-build-a-state-of-the-art-
+           conversational-ai-with-transfer-learning-2d818ac26313
 
-        @param logits: logits tensor
-        @param top_k: keep only top k tokens with highest probability
-        @param top_p: keep the top tokens with cumulative probability
-        @filter_value: value to set filtered tokens to
-        @started: a tensor of bools indicating whether the text generation starts for the batch
-        returns the filtered logits
+     @param logits: logits tensor
+     @param top_k: keep only top k tokens with highest probability
+     @param top_p: keep the top tokens with cumulative probability
+     @filter_value: value to set filtered tokens to
+     @started: a tensor of bools indicating whether the text generation starts for the batch
+     returns the filtered logits
     """
     if top_k > 0:
         # Remove all tokens with a probability less than the
@@ -218,7 +409,7 @@ def top_k_logits(logits, top_k=0, top_p=0.0, filter_value=-float('Inf'), started
         else:
             logits[indices_to_remove] = filter_value
 
-    if top_p > 0.0:
+    if 0.0 < top_p < 1.0:
         # Cconvert to 1D
         sorted_logits, sorted_indices = torch.sort(logits, descending=True, dim=-1)
         cumulative_probs = torch.cumsum(F.softmax(sorted_logits, dim=-1), dim=-1)
@@ -242,7 +433,7 @@ def top_k_logits(logits, top_k=0, top_p=0.0, filter_value=-float('Inf'), started
 
 
 def repetition_penalty(logits, repetition_penalty, used_tokens):
-    """ Implement the repetition penalty, check paper 
+    """Implement the repetition penalty, check paper
     https://arxiv.org/pdf/1909.05858.pdf
     """
     if used_tokens is not None and repetition_penalty != 1.0:
@@ -258,10 +449,15 @@ def get_model_parallel_src_rank():
     all_ranks = np.arange(world_size)
     tp_size = parallel_state.get_tensor_model_parallel_world_size()
     pp_size = parallel_state.get_pipeline_model_parallel_world_size()
-    # [pipeline dim, data parallel, tensor dim]
-    all_ranks = all_ranks.reshape(pp_size, -1, tp_size)
     dp_rank = parallel_state.get_data_parallel_rank()
-    return all_ranks[:, dp_rank, :].min()
+    if AppState().use_tp_pp_dp_mapping:
+        # [DP, PP, TP]
+        all_ranks = all_ranks.reshape(-1, pp_size, tp_size)
+        return all_ranks[dp_rank, :, :].min()
+    else:
+        # [PP, DP, TP]
+        all_ranks = all_ranks.reshape(pp_size, -1, tp_size)
+        return all_ranks[:, dp_rank, :].min()
 
 
 def send_generate_info(
@@ -269,6 +465,7 @@ def send_generate_info(
     context_length_tensor,
     tokens_to_generate,
     all_probs,
+    compute_logprob,
     temperature,
     top_k,
     top_p,
@@ -276,24 +473,29 @@ def send_generate_info(
     repetition_penalty,
     min_tokens_to_generate,
     end_strings,
+    random_seed,
 ):
     """
     Needs to be synced up with receive_generate_info
     """
     model_parallel_group = parallel_state.get_model_parallel_group()
     src = get_model_parallel_src_rank()
+    if random_seed is None:
+        random_seed = -1  # to be able to convert to float
     # Send the sizes of the tensors
     input_info = [
         context_tokens_tensor.size(0),  # batch_size
         context_tokens_tensor.size(1),  # seq_len
         tokens_to_generate,
         all_probs,
+        compute_logprob,  # whether to compute log probabilities matrix
         temperature,
         top_k,
         top_p,
         greedy,
         repetition_penalty,
         min_tokens_to_generate,
+        random_seed,
     ]
     input_info_tensor = torch.cuda.FloatTensor(input_info)
     torch.distributed.broadcast(input_info_tensor, src, model_parallel_group)
@@ -317,18 +519,22 @@ def receive_generate_info():
     """
     model_parallel_group = parallel_state.get_model_parallel_group()
     src = get_model_parallel_src_rank()
-    input_info_tensor = torch.empty(10, dtype=torch.float32, device=torch.cuda.current_device())
+    input_info_tensor = torch.empty(12, dtype=torch.float32, device=torch.cuda.current_device())
     torch.distributed.broadcast(input_info_tensor, src, model_parallel_group)
     batch_size = int(input_info_tensor[0].item())
     seq_len = int(input_info_tensor[1].item())
     tokens_to_generate = int(input_info_tensor[2].item())
     all_probs = bool(input_info_tensor[3].item())
-    temperature = float(input_info_tensor[4].item())
-    top_k = int(input_info_tensor[5].item())
-    top_p = float(input_info_tensor[6].item())
-    greedy = bool(input_info_tensor[7].item())
-    repetition_penalty = float(input_info_tensor[8].item())
-    min_tokens_to_generate = int(input_info_tensor[9].item())
+    compute_logprob = bool(input_info_tensor[4].item())  # whether to compute log probabilities matrix
+    temperature = float(input_info_tensor[5].item())
+    top_k = int(input_info_tensor[6].item())
+    top_p = float(input_info_tensor[7].item())
+    greedy = bool(input_info_tensor[8].item())
+    repetition_penalty = float(input_info_tensor[9].item())
+    min_tokens_to_generate = int(input_info_tensor[10].item())
+    random_seed = int(input_info_tensor[11].item())
+    if random_seed == -1:  # was converted to -1 before broadcast
+        random_seed = None
 
     context_length_tensor = torch.empty(batch_size, dtype=torch.int64, device=torch.cuda.current_device())
     context_tokens_tensor = torch.empty(batch_size, seq_len, dtype=torch.int64, device=torch.cuda.current_device())
@@ -349,6 +555,7 @@ def receive_generate_info():
         context_tokens_tensor,
         tokens_to_generate,
         all_probs,
+        compute_logprob,
         temperature,
         top_k,
         top_p,
@@ -356,6 +563,7 @@ def receive_generate_info():
         repetition_penalty,
         min_tokens_to_generate,
         end_strings,
+        random_seed,
     )
 
 
@@ -370,9 +578,13 @@ def synced_generate(
     top_k=0,
     top_p=0.0,
     greedy=False,
+    compute_attention_mask=True,
+    compute_logprob=False,
     repetition_penalty=1.2,
-    min_tokens_to_generate=0,
     end_strings=[],
+    min_tokens_to_generate=0,
+    image_list=None,
+    **strategy_args,
 ):
     context_length = context_length_tensor.min().item()
     tokenizer = model.tokenizer
@@ -384,9 +596,23 @@ def synced_generate(
             context_length_tensor,
             tokens_to_generate,
             all_probs,
+            compute_attention_mask=compute_attention_mask,
             temperature=temperature,
         )
     else:
+
+        extra = {
+            "top_p": top_p,
+            "top_k": top_k,
+            "greedy": greedy,
+            "repetition_penalty": repetition_penalty,
+            "min_tokens_to_generate": min_tokens_to_generate,
+        }
+
+        # if input containing neighbors (for Mcore retrieval RETRO model)
+        if "neighbors_tokens" in strategy_args:
+            extra['neighbors_tokens'] = strategy_args['neighbors_tokens']
+
         batch_token_iterator = sample_sequence_batch(
             model,
             inference_strategy,
@@ -394,15 +620,12 @@ def synced_generate(
             context_length_tensor,
             tokens_to_generate,
             all_probs,
+            compute_attention_mask=compute_attention_mask,
+            compute_logprob=compute_logprob,
             temperature=temperature,
             end_strings=end_strings,
-            extra={
-                "top_p": top_p,
-                "top_k": top_k,
-                "greedy": greedy,
-                "repetition_penalty": repetition_penalty,
-                "min_tokens_to_generate": min_tokens_to_generate,
-            },
+            image_list=image_list,
+            extra=extra,
         )
 
     for tokens, lengths, output_logits, full_logits in batch_token_iterator:
@@ -411,7 +634,8 @@ def synced_generate(
     if parallel_state.is_pipeline_last_stage():
         src = parallel_state.get_pipeline_model_parallel_last_rank()
         group = parallel_state.get_embedding_group()
-        torch.distributed.broadcast(output_logits, src, group)
+        if compute_logprob:
+            torch.distributed.broadcast(output_logits, src, group)
         if all_probs:
             src = parallel_state.get_pipeline_model_parallel_last_rank()
             group = parallel_state.get_embedding_group()
@@ -421,10 +645,15 @@ def synced_generate(
         if parallel_state.is_pipeline_first_stage():
             src = parallel_state.get_pipeline_model_parallel_last_rank()
             group = parallel_state.get_embedding_group()
-            output_logits = torch.empty(
-                tokens.size(0), context_length - 1, dtype=torch.float32, device=torch.device("cuda")
-            )
-            torch.distributed.broadcast(output_logits, src, group)
+
+            if compute_logprob:
+                precision = model._trainer.precision
+                dtype = torch.float32
+
+                output_logits = torch.empty(
+                    tokens.size(0), context_length - 1, dtype=dtype, device=torch.device("cuda")
+                )
+                torch.distributed.broadcast(output_logits, src, group)
 
             if all_probs:
                 src = parallel_state.get_pipeline_model_parallel_last_rank()
@@ -433,7 +662,7 @@ def synced_generate(
                     tokens.size(0),
                     context_length - 1,
                     model.padded_vocab_size,
-                    dtype=torch.float32,
+                    dtype=dtype,
                     device=torch.device("cuda"),
                 )
                 torch.distributed.broadcast(full_logits, src, group)
@@ -451,9 +680,13 @@ def generate(
     top_k=0,
     top_p=0.0,
     greedy=False,
+    compute_attention_mask=True,
+    compute_logprob=False,
     repetition_penalty=1.0,
-    min_tokens_to_generate=0,
     end_strings=['<|endoftext|>'],
+    image_list=None,
+    min_tokens_to_generate=0,
+    random_seed=None,
     **strategy_args,
 ) -> OutputType:
     """
@@ -469,10 +702,14 @@ def generate(
         greedy (bool):  Whether or not to use sampling ; use greedy decoding otherwise
         repetition_penalty (float): The parameter for repetition penalty. 1.0 means no penalty
         min_tokens_to_generate (int): The minimum length of the tokens to be generated
+        random_seed (int): can set to fix random seed for reproducibility. If None, we do not set random seed, so
+            the behavior of generation will depend on whether the seed was set earlier or not.
         strategy_args, the extra arguments are treated as inference strategy arguments
         end_strings, a list of strings to stop generation when they are encountered in the output.
+
     Returns:
         OutputType: It generates the output in a dictionary type. It has the following keys:
+
             sentences: List[str], output sentences
             tokens: List[List[str]], output sentences borken into tokens
             logprob: List[Tensor], log prob of generated tokens
@@ -498,6 +735,7 @@ def generate(
             context_length_tensor,
             tokens_to_generate,
             all_probs,
+            compute_logprob,
             temperature,
             top_k,
             top_p,
@@ -505,13 +743,31 @@ def generate(
             repetition_penalty,
             min_tokens_to_generate,
             end_strings,
+            random_seed,
         )
+
+        # tokenize neighbors and broadcast (for Mcore retrieval RETRO model)
+        if 'neighbors' in strategy_args:
+            # tokenize neighbors
+            neighbors_tokens_tensor, neighbors_tokens_tensor_shape = inference_strategy.tokenize_neighbors_batch(
+                strategy_args['neighbors'], strategy_args['retro_inference']
+            )
+
+            # send neighbors tensors to all ranks
+            model_parallel_group = parallel_state.get_model_parallel_group()
+            src = get_model_parallel_src_rank()
+            torch.distributed.broadcast(neighbors_tokens_tensor_shape, src, model_parallel_group)
+            torch.distributed.broadcast(neighbors_tokens_tensor, src, model_parallel_group)
+        else:
+            neighbors_tokens_tensor = None
+
     else:
         (
             context_length_tensor,
             context_tokens_tensor,
             tokens_to_generate,
             all_probs,
+            compute_logprob,
             temperature,
             top_k,
             top_p,
@@ -519,8 +775,36 @@ def generate(
             repetition_penalty,
             min_tokens_to_generate,
             end_strings,
+            random_seed,
         ) = receive_generate_info()
 
+        # receive broadcast (for Mcore retrieval RETRO model)
+        if 'neighbors' in strategy_args:
+            # receive neighbors tensors to all ranks
+            model_parallel_group = parallel_state.get_model_parallel_group()
+            src = get_model_parallel_src_rank()
+            neighbors_tokens_tensor_shape = torch.empty(2, dtype=torch.float32, device=torch.cuda.current_device())
+            torch.distributed.broadcast(neighbors_tokens_tensor_shape, src, model_parallel_group)
+            neighbors_tokens_tensor = torch.empty(
+                neighbors_tokens_tensor_shape[0],
+                neighbors_tokens_tensor_shape[1],
+                dtype=torch.int64,
+                device=torch.cuda.current_device(),
+            )
+            torch.distributed.broadcast(neighbors_tokens_tensor, src, model_parallel_group)
+        else:
+            neighbors_tokens_tensor = None
+
+    # add neighbors to strategy_args (for retrieval RETRO model)
+    if 'neighbors' in strategy_args:
+        strategy_args['neighbors_tokens'] = neighbors_tokens_tensor
+
+    if random_seed is not None:
+        seed_everything(random_seed)
+
+    if hasattr(model, 'get_attention_mask_from_fusion') and model.get_attention_mask_from_fusion:
+        compute_attention_mask = False
+
     output = synced_generate(
         model,
         inference_strategy,
@@ -529,12 +813,16 @@ def generate(
         tokens_to_generate,
         all_probs,
         temperature,
+        compute_attention_mask=compute_attention_mask,
+        compute_logprob=compute_logprob,
         top_k=top_k,
         top_p=top_p,
         greedy=greedy,
         repetition_penalty=repetition_penalty,
-        min_tokens_to_generate=min_tokens_to_generate,
         end_strings=end_strings,
+        min_tokens_to_generate=min_tokens_to_generate,
+        image_list=image_list,
+        **strategy_args,
     )
     special_tokens = set()
     if hasattr(tokenizer, 'pad_token') and tokenizer.pad_token is not None:
@@ -613,25 +901,25 @@ def sample_sequence_batch(
     context_lengths,
     tokens_to_generate,
     all_probs=False,
+    compute_attention_mask=True,
+    compute_logprob=False,
     type_ids=None,
     temperature=None,
     end_strings=['<|endoftext|>'],
+    image_list=None,
     extra={},
 ):
     # Importing here to avoid circular import errors
 
     app_state = AppState()
     micro_batch_size = context_tokens.shape[0]
-    _reconfigure_microbatch_calculator(
+    reconfigure_num_microbatches_calculator(
         rank=app_state.global_rank,
         rampup_batch_size=None,
         global_batch_size=micro_batch_size,
         micro_batch_size=micro_batch_size,
         data_parallel_size=1,
     )
-    assert (
-        model.cfg.get('sequence_parallel', False) == False
-    ), 'sequence_parallel should be False during inference. Disable it in the model config if restoring from nemo or in hparams.yaml if restoring from PTL checkpoint'
     assert (
         model.cfg.get('activations_checkpoint_granularity', None) is None
     ), 'activations_checkpoint_granularity should be None during inference. Disable it in the model config if restoring from nemo or in hparams.yaml if restoring from PTL checkpoint'
@@ -643,7 +931,15 @@ def sample_sequence_batch(
     # initialize the batch
     with torch.no_grad():
         context_length = context_lengths.min().item()
-        inference_strategy.init_batch(context_tokens, context_length)
+        if 'neighbors_tokens' in extra:  # for Mcore retrieval RETRO model
+
+            # For Mcore retrieval RETRO model, context_tokens tensors are updated after init_batch() (the length is doubled after processing)
+            context_tokens = inference_strategy.init_batch(
+                context_tokens, context_length, compute_attention_mask, **extra
+            )
+
+        else:
+            inference_strategy.init_batch(context_tokens, context_length, compute_attention_mask)
         # added eos_id to support the function generate_samples_eval that passes
         # eos_id as an argument and needs termination when that id id found.
         eod_id = tokenizer.eos_id
@@ -660,18 +956,35 @@ def sample_sequence_batch(
         maxlen = inference_strategy.clip_max_len(maxlen)
 
         lengths = torch.ones([batch_size]).long().cuda() * maxlen
+
         while context_length < maxlen:
-            batch, tensor_shape = inference_strategy.prepare_batch_at_step(
-                tokens, maxlen, micro_batch_size, counter, context_length
-            )
+            if image_list is not None:
+                batch, tensor_shape = inference_strategy.prepare_batch_at_step(
+                    tokens, maxlen, micro_batch_size, counter, context_length, compute_attention_mask, image_list
+                )
+            else:
+                batch, tensor_shape = inference_strategy.prepare_batch_at_step(
+                    tokens, maxlen, micro_batch_size, counter, context_length, compute_attention_mask
+                )
             output = inference_strategy.forward_step(batch, tensor_shape)
 
             if parallel_state.is_pipeline_last_stage():
-                output = output[0]['logits'].float()
-                output = tensor_parallel.gather_from_tensor_model_parallel_region(output)
-                assert output is not None
-                output = output.float()
-                logits = output[:, -1].view(batch_size, -1).contiguous()
+
+                if compute_logprob:
+                    output = output[0]['logits']
+                    output = tensor_parallel.gather_from_tensor_model_parallel_region(output)
+                    assert output is not None
+                    logits = output[:, -1].view(batch_size, -1).contiguous()
+
+                else:
+                    if 'neighbors_tokens' in extra:  # for Mcore retrieval RETRO model
+                        # for Mcore RETRO inference, disimilar to GPT, we will get the logits of the (context_length - 1)th token, instead of the last token
+                        logits = output[0]['logits'][:, context_length - 1].contiguous()
+                    else:
+                        logits = output[0]['logits'][:, -1].contiguous()
+                    logits = tensor_parallel.gather_from_tensor_model_parallel_region(logits)
+                    assert logits is not None
+                    logits = logits.view(batch_size, -1)
 
                 # make sure it will generate at least min_length
                 min_length = extra.get('min_tokens_to_generate', 0)
@@ -683,6 +996,7 @@ def sample_sequence_batch(
                 logits[:, tokenizer.vocab_size :] = -float('Inf')
 
                 # started indicates whether the current token step passes the context_length, so we make sure not to overwrite the context tokens
+
                 started = context_lengths <= context_length
                 if extra.get('greedy', False):
                     prev = torch.argmax(logits, dim=-1).view(-1)
@@ -710,23 +1024,25 @@ def sample_sequence_batch(
                 # Insert either new predicted or next prompt token
                 tokens[:, context_length] = new_tokens
 
-                if output_logits is None:
-                    output = F.log_softmax(output[:, :context_length, :], 2)
-                    indices = torch.unsqueeze(tokens[:, 1 : context_length + 1], 2)
-                    output_logits = torch.gather(output, 2, indices).squeeze(2)
-                    all_generated_indices = indices[:, :, 0]
-                    if all_probs:
-                        full_logits = output
-                else:
-                    output = F.log_softmax(output, 2)
-                    indices = torch.unsqueeze(new_tokens, 1).unsqueeze(2)
-                    new_output_logits = torch.gather(output, 2, indices).squeeze(2)
+                if compute_logprob:
+                    if output_logits is None:
+                        output = F.log_softmax(output[:, :context_length, :], 2)
 
-                    # TODO(rprenger) we're copying output_logits every time.  Should pre-allocate
-                    output_logits = torch.cat([output_logits, new_output_logits], 1)
-                    all_generated_indices = torch.cat([all_generated_indices, indices[:, :, 0]], 1)
-                    if all_probs:
-                        full_logits = torch.cat([full_logits, output], 1)
+                        indices = torch.unsqueeze(tokens[:, 1 : context_length + 1], 2)
+                        output_logits = torch.gather(output, 2, indices).squeeze(2)
+                        all_generated_indices = indices[:, :, 0]
+                        if all_probs:
+                            full_logits = output
+                    else:
+                        output = F.log_softmax(output, 2)
+                        indices = torch.unsqueeze(new_tokens, 1).unsqueeze(2)
+                        new_output_logits = torch.gather(output, 2, indices).squeeze(2)
+
+                        # TODO(rprenger) we're copying output_logits every time.  Should pre-allocate
+                        output_logits = torch.cat([output_logits, new_output_logits], 1)
+                        all_generated_indices = torch.cat([all_generated_indices, indices[:, :, 0]], 1)
+                        if all_probs:
+                            full_logits = torch.cat([full_logits, output], 1)
 
                 src = parallel_state.get_pipeline_model_parallel_last_rank()
                 group = parallel_state.get_embedding_group()
@@ -746,10 +1062,13 @@ def sample_sequence_batch(
                 src = parallel_state.get_pipeline_model_parallel_last_rank()
                 group = parallel_state.get_pipeline_model_parallel_group()
                 torch.distributed.broadcast(done, src, group)
-                if all_probs:
-                    yield tokens, lengths, output_logits, full_logits
+                if compute_logprob:
+                    if all_probs:
+                        yield tokens, lengths, output_logits, full_logits
+                    else:
+                        yield tokens, lengths, output_logits, None
                 else:
-                    yield tokens, lengths, output_logits, None
+                    yield tokens, lengths, None, None
 
             else:
                 if parallel_state.is_pipeline_first_stage():
@@ -780,12 +1099,13 @@ def tab_sample_sequence_batch(
     context_lengths,
     tokens_to_generate,
     all_probs=True,
+    compute_attention_mask=True,
     type_ids=None,
     temperature=None,
 ):
     app_state = AppState()
     micro_batch_size = context_tokens.shape[0]
-    _reconfigure_microbatch_calculator(
+    reconfigure_num_microbatches_calculator(
         rank=app_state.global_rank,
         rampup_batch_size=None,
         global_batch_size=micro_batch_size,
@@ -803,7 +1123,7 @@ def tab_sample_sequence_batch(
     # initialize the batch
     with torch.no_grad():
         context_length = context_lengths.min().item()
-        inference_strategy.init_batch(context_tokens, context_length)
+        inference_strategy.init_batch(context_tokens, context_length, compute_attention_mask)
         context = context_tokens[:, :context_length]
         # the context may start in the middle of the row,
         # calculate the offset according to the position of '\n' or '<|endoftext|>'
@@ -837,7 +1157,7 @@ def tab_sample_sequence_batch(
 
         while context_length < maxlen:
             batch, tensor_shape = inference_strategy.prepare_batch_at_step(
-                tokens, maxlen, micro_batch_size, counter, context_length
+                tokens, maxlen, micro_batch_size, counter, context_length, compute_attention_mask
             )
             output = inference_strategy.forward_step(batch, tensor_shape)
 
@@ -936,7 +1256,7 @@ def sample_token_greedy(logits):
 
     Args:
         logits: [batch_size, vocab_size] - unnormalized log probabilities of the next token
-    
+
     Returns:
         log_probs: [batch_size] - log probabilities of the sampled tokens
         token_ids: [batch_size] - sampled token ids
@@ -956,7 +1276,7 @@ def sample_token_topk(logits, top_k=0, top_p=0.0, temperature=1.0, filter_value=
         top_p: float - if > 0.0: only sample from a subset of candidates, where the cumulative probability
         temperature: float - temperature for sampling
         filter_value: float - value to set filtered tokens to
-    
+
     Returns:
         log_probs: [batch_size] - log probabilities of the sampled tokens
         token_ids: [batch_size] - sampled token ids
