@@ -13,9 +13,11 @@
 # limitations under the License.
 
 from nemo.collections.tts.models.aligner import AlignerModel
+from nemo.collections.tts.models.audio_codec import AudioCodecModel
 from nemo.collections.tts.models.fastpitch import FastPitchModel
 from nemo.collections.tts.models.fastpitch_ssl import FastPitchModel_SSL
 from nemo.collections.tts.models.hifigan import HifiGanModel
+from nemo.collections.tts.models.magpietts import MagpieTTS_Model, MagpieTTS_ModelDPO, MagpieTTS_ModelInference
 from nemo.collections.tts.models.mixer_tts import MixerTTSModel
 from nemo.collections.tts.models.radtts import RadTTSModel
 from nemo.collections.tts.models.spectrogram_enhancer import SpectrogramEnhancerModel
@@ -28,6 +30,7 @@ from nemo.collections.tts.models.waveglow import WaveGlowModel
 
 __all__ = [
     "AlignerModel",
+    "AudioCodecModel",
     "FastPitchModel",
     "FastPitchModel_SSL",
     "SSLDisentangler",
@@ -36,6 +39,9 @@ __all__ = [
     "MelPsuedoInverseModel",
     "MixerTTSModel",
     "RadTTSModel",
+    "MagpieTTS_Model",
+    "MagpieTTS_ModelInference",
+    "MagpieTTS_ModelDPO",
     "Tacotron2Model",
     "TwoStagesModel",
     "UnivNetModel",
