@@ -41,26 +41,33 @@ from typing import List, Optional
 
 import torch
 import torch.nn.functional as F
-import torchvision
 from einops import rearrange
 from hydra.utils import instantiate
+from lightning.pytorch import Trainer
+from lightning.pytorch.loggers import TensorBoardLogger, WandbLogger
 from omegaconf import DictConfig
-from pytorch_lightning import Trainer
-from pytorch_lightning.loggers import TensorBoardLogger, WandbLogger
 from torch.utils.tensorboard.writer import SummaryWriter
 
+from nemo.collections.common.parts.utils import mask_sequence_tensor
 from nemo.collections.tts.losses.spectrogram_enhancer_losses import (
     ConsistencyLoss,
     GeneratorLoss,
     GradientPenaltyLoss,
     HingeLoss,
 )
-from nemo.collections.tts.parts.utils.helpers import mask_sequence_tensor, to_device_recursive
-from nemo.core import Exportable, ModelPT, typecheck
+from nemo.collections.tts.parts.utils.helpers import to_device_recursive
+from nemo.core import Exportable, ModelPT, PretrainedModelInfo, typecheck
 from nemo.core.neural_types import LengthsType, MelSpectrogramType, NeuralType
 from nemo.core.neural_types.elements import BoolType
 from nemo.utils import logging
 
+try:
+    import torchvision
+
+    TORCHVISION_AVAILABLE = True
+except (ImportError, ModuleNotFoundError):
+    TORCHVISION_AVAILABLE = False
+
 
 class SpectrogramEnhancerModel(ModelPT, Exportable):
     """
@@ -122,7 +129,12 @@ class SpectrogramEnhancerModel(ModelPT, Exportable):
         }
     )
     def forward(
-        self, *, input_spectrograms: torch.Tensor, lengths: torch.Tensor, mixing: bool = False, normalize: bool = True,
+        self,
+        *,
+        input_spectrograms: torch.Tensor,
+        lengths: torch.Tensor,
+        mixing: bool = False,
+        normalize: bool = True,
     ):
         """
         Generator forward pass. Noise inputs will be generated.
@@ -257,7 +269,10 @@ class SpectrogramEnhancerModel(ModelPT, Exportable):
             return g_loss + c_loss
 
     def configure_optimizers(self):
-        generator_opt = instantiate(self._cfg.generator_opt, params=self.generator.parameters(),)
+        generator_opt = instantiate(
+            self._cfg.generator_opt,
+            params=self.generator.parameters(),
+        )
         discriminator_opt = instantiate(self._cfg.discriminator_opt, params=self.discriminator.parameters())
         return [discriminator_opt, generator_opt], []
 
@@ -277,7 +292,23 @@ class SpectrogramEnhancerModel(ModelPT, Exportable):
 
     @classmethod
     def list_available_models(cls):
-        return []
+        list_of_models = []
+
+        # en, multi speaker, LibriTTS, 16000 Hz
+        # stft 25ms 10ms matching ASR params
+        # for use during Enhlish ASR training/adaptation
+        model = PretrainedModelInfo(
+            pretrained_model_name="tts_en_spectrogram_enhancer_for_asr_finetuning",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/tts_en_fastpitch_spectrogram_enhancer_for_asr_finetuning/versions/1.20.0/files/tts_en_spectrogram_enhancer_for_asr_finetuning.nemo",
+            description="This model is trained to add details to synthetic spectrograms."
+            " It was trained on pairs of real-synthesized spectrograms generated by FastPitch."
+            " STFT parameters follow ASR with 25 ms window and 10 ms hop."
+            " It is supposed to be used in conjunction with that model for ASR training/adaptation.",
+            class_=cls,
+        )
+        list_of_models.append(model)
+
+        return list_of_models
 
     def log_illustration(self, target_spectrograms, input_spectrograms, enhanced_spectrograms, lengths):
         if self.global_rank != 0:
@@ -302,6 +333,7 @@ class SpectrogramEnhancerModel(ModelPT, Exportable):
             dim=0,
         ).cpu()[:, idx, :, :, :length]
 
+        assert TORCHVISION_AVAILABLE, "Torchvision imports failed but they are required."
         grid = torchvision.utils.make_grid(tensor, nrow=1).clamp(0.0, 1.0)
 
         for logger in self.loggers:
