@@ -14,25 +14,32 @@
 
 
 """Interfaces common to all Neural Modules and Models."""
+import copy
 import hashlib
 import inspect
+import os
+import shutil
 import traceback
 from abc import ABC, abstractmethod
+from collections.abc import Mapping, Sequence
 from contextlib import contextmanager
 from dataclasses import dataclass, field
 from enum import Enum
 from functools import total_ordering
 from pathlib import Path
-from typing import Dict, Iterable, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import hydra
 import torch
 import wrapt
-from huggingface_hub import HfApi, HfFolder, ModelFilter, hf_hub_download
-from huggingface_hub.hf_api import ModelInfo
+from huggingface_hub import HfApi
+from huggingface_hub import get_token as get_hf_token
+from huggingface_hub import hf_hub_download, snapshot_download
 from omegaconf import DictConfig, OmegaConf
 
 import nemo
+from nemo.core.classes.mixins.hf_io_mixin import HuggingFaceFileIO
+from nemo.core.config.templates.model_card import NEMO_DEFAULT_MODEL_CARD_TEMPLATE
 from nemo.core.connectors.save_restore_connector import SaveRestoreConnector
 from nemo.core.neural_types import NeuralType, NeuralTypeComparisonResult
 from nemo.utils import logging
@@ -43,10 +50,63 @@ from nemo.utils.model_utils import import_class_by_path, maybe_update_config_ver
 __all__ = ['Typing', 'FileIO', 'Model', 'Serialization', 'typecheck', 'PretrainedModelInfo']
 
 _TYPECHECK_ENABLED = True
+_TYPECHECK_SEMANTIC_CHECK_ENABLED = True
 # TODO @blisc: Remove _HAS_HYDRA
 _HAS_HYDRA = True
 
 
+# Added these for now but these should be updated based on collections
+ALLOWED_TARGET_PREFIXES = [
+    "nemo.collections.",
+    "nemo.core.",
+    "nemo.utils.",
+    "nemo.lightning.",
+    "tests.collections.",
+    "torch.nn.",
+    "torch.optim.",
+    "torch.utils.data.",
+    "lightning.pytorch.callbacks.",
+    "lightning.pytorch.loggers.",
+    "lightning.pytorch.strategies.",
+    "lightning.pytorch.accelerators.",
+    "omegaconf.",
+    "megatron.",
+]
+
+
+def _is_target_allowed(target_path: str) -> bool:
+    if not isinstance(target_path, str):
+        return False
+    return any(target_path.startswith(prefix) for prefix in ALLOWED_TARGET_PREFIXES)
+
+
+def _validate_config_targets_recursive(config_node: Any):
+    if isinstance(config_node, Mapping):  # Handles DictConfig and dict
+        if "_target_" in config_node:
+            target_path = config_node["_target_"]
+            if not _is_target_allowed(target_path):
+                raise ValueError(
+                    f"Instantiation of unsafe target '{target_path}' is blocked. "
+                    f"The '_target_' must point to a class or function within an approved namespace. "
+                    f"This restriction is in place to prevent potential arbitrary code execution."
+                )
+        for key, value in config_node.items():
+            _validate_config_targets_recursive(value)
+    elif isinstance(config_node, Sequence) and not isinstance(config_node, str):  # Handles ListConfig and list
+        for item in config_node:
+            _validate_config_targets_recursive(item)
+
+
+def safe_instantiate(config: DictConfig, *args, **kwargs):
+    """
+    A wrapper around hydra.utils.instantiate that first validates all _target_
+    fields in the config against an allow-list of prefixes.
+    """
+    if config is not None:
+        _validate_config_targets_recursive(config)
+    return hydra.utils.instantiate(config, *args, **kwargs)
+
+
 def is_typecheck_enabled():
     """
     Getter method for typechecking state.
@@ -54,6 +114,13 @@ def is_typecheck_enabled():
     return _TYPECHECK_ENABLED
 
 
+def is_semantic_typecheck_enabled():
+    """
+    Getter method for typechecking semantics state.
+    """
+    return _TYPECHECK_SEMANTIC_CHECK_ENABLED
+
+
 @dataclass
 class TypecheckMetadata:
     """
@@ -119,7 +186,7 @@ class TypecheckMetadata:
                 while isinstance(type_val, (list, tuple)):
                     if len(type_val) > 1:
                         raise TypeError(
-                            f"Neural Type `{type_key}`: {type_val} definition contains more than one element when"
+                            f"Neural Type `{type_key}`: {type_val} definition contains more than one element when "
                             "declaring the nested container structure.\n"
                             "Please ensure that you have only 1 NeuralType inside of the entire nested structure "
                             "definition."
@@ -178,7 +245,6 @@ class Typing(ABC):
             kwargs: Dictionary of argument_name:argument_value pairs passed to the wrapped
                 function upon call.
         """
-        # TODO: Properly implement this
         if input_types is not None:
             # Precompute metadata
             metadata = TypecheckMetadata(original_types=input_types, ignore_collections=ignore_collections)
@@ -202,9 +268,14 @@ class Typing(ABC):
                     )
 
                 # Perform neural type check
-                if hasattr(value, 'neural_type') and not metadata.base_types[key].compare(value.neural_type) in (
-                    NeuralTypeComparisonResult.SAME,
-                    NeuralTypeComparisonResult.GREATER,
+                if (
+                    hasattr(value, 'neural_type')
+                    and is_semantic_typecheck_enabled()
+                    and not metadata.base_types[key].compare(value.neural_type)
+                    in (
+                        NeuralTypeComparisonResult.SAME,
+                        NeuralTypeComparisonResult.GREATER,
+                    )
                 ):
                     error_msg = [
                         f"{input_types[key].compare(value.neural_type)} :",
@@ -379,9 +450,14 @@ class Typing(ABC):
                 f"Expected nested depth : {metadata.container_depth[name]}"
             )
 
-        if hasattr(obj, 'neural_type') and not type_val.compare(obj.neural_type) in (
-            NeuralTypeComparisonResult.SAME,
-            NeuralTypeComparisonResult.GREATER,
+        if (
+            hasattr(obj, 'neural_type')
+            and is_semantic_typecheck_enabled()
+            and not type_val.compare(obj.neural_type)
+            in (
+                NeuralTypeComparisonResult.SAME,
+                NeuralTypeComparisonResult.GREATER,
+            )
         ):
             raise TypeError(
                 f"{type_val.compare(obj.neural_type)} : \n"
@@ -461,21 +537,21 @@ class Serialization(ABC):
         # Hydra 0.x API
         if ('cls' in config or 'target' in config) and 'params' in config and _HAS_HYDRA:
             # regular hydra-based instantiation
-            instance = hydra.utils.instantiate(config=config)
+            instance = safe_instantiate(config=config)
         # Hydra 1.x API
         elif '_target_' in config and _HAS_HYDRA:
             # regular hydra-based instantiation
-            instance = hydra.utils.instantiate(config=config)
+            instance = safe_instantiate(config=config)
         else:
             instance = None
             prev_error = ""
             # Attempt class path resolution from config `target` class (if it exists)
             if 'target' in config:
-                target_cls = config["target"]  # No guarantee that this is a omegaconf class
+                target_cls_path = config["target"]  # No guarantee that this is a omegaconf class
                 imported_cls = None
                 try:
                     # try to import the target class
-                    imported_cls = import_class_by_path(target_cls)
+                    imported_cls = import_class_by_path(target_cls_path)
                     # if calling class (cls) is subclass of imported class,
                     # use subclass instead
                     if issubclass(cls, imported_cls):
@@ -488,7 +564,9 @@ class Serialization(ABC):
                 except Exception as e:
                     # record previous error
                     tb = traceback.format_exc()
-                    prev_error = f"Model instantiation failed!\nTarget class:\t{target_cls}" f"\nError(s):\t{e}\n{tb}"
+                    prev_error = (
+                        f"Model instantiation failed!\nTarget class:\t{target_cls_path}" f"\nError(s):\t{e}\n{tb}"
+                    )
                     logging.debug(prev_error + "\nFalling back to `cls`.")
 
             # target class resolution was unsuccessful, fall back to current `cls`
@@ -652,7 +730,7 @@ class PretrainedModelInfo:
         return self.pretrained_model_name < other.pretrained_model_name
 
 
-class Model(Typing, Serialization, FileIO):
+class Model(Typing, Serialization, FileIO, HuggingFaceFileIO):
     """
     Abstract class offering interface which should be implemented by all NeMo models.
     """
@@ -662,113 +740,14 @@ class Model(Typing, Serialization, FileIO):
     def list_available_models(cls) -> Optional[List[PretrainedModelInfo]]:
         """
         Should list all pre-trained models available via NVIDIA NGC cloud.
-        Note: There is no check that requires model names and aliases to be unique. In the case of a collision, whatever
-        model (or alias) is listed first in the this returned list will be instantiated.
+        Note: There is no check that requires model names and aliases to be unique. In the case of a collision,
+        whatever model (or alias) is listed first in the this returned list will be instantiated.
 
         Returns:
             A list of PretrainedModelInfo entries
         """
         pass
 
-    @classmethod
-    def search_huggingface_models(
-        cls, model_filter: Optional[Union[ModelFilter, List[ModelFilter]]] = None
-    ) -> List[ModelInfo]:
-        """
-        Should list all pre-trained models available via Hugging Face Hub.
-
-        The following metadata can be passed via the `model_filter` for additional results.
-        Metadata:
-            resolve_card_info: Bool flag, if set, returns the model card metadata. Default: False.
-            limit_results: Optional int, limits the number of results returned.
-
-        .. code-block:: python
-
-            # You can replace <DomainSubclass> with any subclass of ModelPT.
-            from nemo.core import ModelPT
-
-            # Get default ModelFilter
-            filt = <DomainSubclass>.get_hf_model_filter()
-
-            # Make any modifications to the filter as necessary
-            filt.language = [...]
-            filt.task = ...
-            filt.tags = [...]
-
-            # Add any metadata to the filter as needed
-            filt.limit_results = 5
-
-            # Obtain model info
-            model_infos = <DomainSubclass>.search_huggingface_models(model_filter=filt)
-
-            # Browse through cards and select an appropriate one
-            card = model_infos[0]
-
-            # Restore model using `modelId` of the card.
-            model = ModelPT.from_pretrained(card.modelId)
-
-        Args:
-            model_filter: Optional ModelFilter or List[ModelFilter] (from Hugging Face Hub)
-                that filters the returned list of compatible model cards, and selects all results from each filter.
-                Users can then use `model_card.modelId` in `from_pretrained()` to restore a NeMo Model.
-                If no ModelFilter is provided, uses the classes default filter as defined by `get_hf_model_filter()`.
-
-        Returns:
-            A list of ModelInfo entries.
-        """
-        # Resolve model filter if not provided as argument
-        if model_filter is None:
-            model_filter = cls.get_hf_model_filter()
-
-        # If single model filter, wrap into list
-        if not isinstance(model_filter, Iterable):
-            model_filter = [model_filter]
-
-        # Inject `nemo` library filter
-        for mfilter in model_filter:
-            if isinstance(mfilter.library, str) and mfilter.library != 'nemo':
-                logging.warning(f"Model filter's `library` tag updated be `nemo`. Original value: {mfilter.library}")
-                mfilter.library = "nemo"
-
-            elif isinstance(mfilter, Iterable) and 'nemo' not in mfilter.library:
-                logging.warning(
-                    f"Model filter's `library` list updated to include `nemo`. Original value: {mfilter.library}"
-                )
-                mfilter.library = list(mfilter)
-                mfilter.library.append('nemo')
-
-        # Check if api token exists, use if it does
-        is_token_available = HfFolder.get_token() is not None
-
-        # Search for all valid models after filtering
-        api = HfApi()
-
-        # Setup extra arguments for model filtering
-        all_results = []  # type: List[ModelInfo]
-
-        for mfilter in model_filter:
-            cardData = None
-            limit = None
-
-            if hasattr(mfilter, 'resolve_card_info') and mfilter.resolve_card_info is True:
-                cardData = True
-
-            if hasattr(mfilter, 'limit_results') and mfilter.limit_results is not None:
-                limit = mfilter.limit_results
-
-            results = api.list_models(
-                filter=mfilter,
-                use_auth_token=is_token_available,
-                sort="lastModified",
-                direction=-1,
-                cardData=cardData,
-                limit=limit,
-            )  # type: List[ModelInfo]
-
-            all_results.extend(results)
-
-        return all_results
-
     @classmethod
     def get_available_model_names(cls) -> List[str]:
         """
@@ -782,28 +761,6 @@ class Model(Typing, Serialization, FileIO):
             model_names = [model.pretrained_model_name for model in cls.list_available_models()]
         return model_names
 
-    @classmethod
-    def get_hf_model_filter(cls) -> ModelFilter:
-        """
-        Generates a filter for HuggingFace models.
-
-        Additionally includes default values of some metadata about results returned by the Hub.
-
-        Metadata:
-            resolve_card_info: Bool flag, if set, returns the model card metadata. Default: False.
-            limit_results: Optional int, limits the number of results returned.
-
-        Returns:
-            A Hugging Face Hub ModelFilter object.
-        """
-        model_filter = ModelFilter(library='nemo')
-
-        # Attach some additional info
-        model_filter.resolve_card_info = False
-        model_filter.limit_results = None
-
-        return model_filter
-
     @classmethod
     def from_pretrained(
         cls,
@@ -815,6 +772,7 @@ class Model(Typing, Serialization, FileIO):
         return_config: bool = False,
         trainer: Optional['Trainer'] = None,
         save_restore_connector: SaveRestoreConnector = None,
+        return_model_file: Optional[bool] = False,
     ):
         """
         Instantiates an instance of NeMo from NVIDIA NGC cloud
@@ -830,6 +788,7 @@ class Model(Typing, Serialization, FileIO):
             strict: Passed to torch.load_state_dict. By default true.
             return_config: If set to true, will return just the underlying config of the restored
                 model as an OmegaConf DictConfig object without instantiating the model.
+            return_model_file: If set to true, will return just the downloaded model file in cache
 
         Returns:
             A model instance of a particular model class or its underlying config (if return_config is set).
@@ -843,12 +802,21 @@ class Model(Typing, Serialization, FileIO):
             class_, nemo_model_file_in_cache = cls._get_hf_hub_pretrained_model_info(
                 model_name=model_name, refresh_cache=refresh_cache
             )
+
+            # Check if nemo_model_file_in_cache is a directory
+            if os.path.isdir(nemo_model_file_in_cache):
+                # Update SaveRestoreConnector with the flag to read from an unpacked NeMo folder
+                save_restore_connector.model_extracted_dir = nemo_model_file_in_cache
+
         else:
             # NGC source
             class_, nemo_model_file_in_cache = cls._get_ngc_pretrained_model_info(
                 model_name=model_name, refresh_cache=refresh_cache
             )
 
+        if return_model_file:
+            return nemo_model_file_in_cache
+
         instance = class_.restore_from(
             restore_path=nemo_model_file_in_cache,
             override_config_path=override_config_path,
@@ -898,7 +866,8 @@ class Model(Typing, Serialization, FileIO):
 
         if location_in_the_cloud is None:
             raise FileNotFoundError(
-                f"Model {model_name} was not found. Check cls.list_available_models() for the list of all available models."
+                f"Model {model_name} was not found. Check cls.list_available_models()\n"
+                f"for the list of all available models."
             )
         filename = location_in_the_cloud.split("/")[-1]
         url = location_in_the_cloud.replace(filename, "")
@@ -941,17 +910,69 @@ class Model(Typing, Serialization, FileIO):
         resolved_model_filename = model_name.split("/")[-1] + '.nemo'
 
         # Check if api token exists, use if it does
-        is_token_available = HfFolder.get_token() is not None
+        hf_token = get_hf_token()
 
-        # Try to load the model from the Huggingface Hub
-        path = hf_hub_download(
-            repo_id=model_name,
-            filename=resolved_model_filename,
-            library_name="nemo",
-            library_version=nemo.__version__,
-            force_download=refresh_cache,
-            use_auth_token=is_token_available,
-        )
+        # First check if .nemo file exists in HF
+        api = HfApi(token=hf_token)
+
+        # Check if model exists in HF
+        nemo_file_exists = api.file_exists(repo_id=model_name, filename=resolved_model_filename, repo_type="model")
+
+        if nemo_file_exists:
+            # Try to load the model from the Huggingface Hub
+            path = hf_hub_download(
+                repo_id=model_name,
+                filename=resolved_model_filename,
+                library_name='nemo',
+                library_version=nemo.__version__,
+                force_download=refresh_cache,
+                token=hf_token,
+            )
+        else:
+            repo_info = api.repo_info(repo_id=model_name, token=hf_token, files_metadata=True)
+
+            # Download whole HF repo and load entire directory as nemo directory
+            cache_dir = Path.joinpath(resolve_cache_dir(), "hf_hub_cache", f'{model_name}')
+
+            # If either description and location in the cloud changes, this will force re-download
+            cache_subfolder = []
+            # Calculate hash of repo_info
+            for sibling in repo_info.siblings:
+                filename = sibling.rfilename.lower()
+                # Ignore updates to readme when downloading hash
+                if "readme" not in filename or "git" not in filename:
+                    cache_subfolder.append(sibling.blob_id)
+            cache_subfolder = sorted(cache_subfolder)
+            cache_subfolder = "".join(cache_subfolder)
+            cache_subfolder = hashlib.md5(cache_subfolder.encode('utf-8')).hexdigest()
+
+            # if file exists on cache_folder/subfolder, it will be re-used, unless refresh_cache is True
+            save_path = os.path.join(cache_dir, cache_subfolder)
+
+            # If the cache dir already exists, delete it to preserve disk space
+            if os.path.exists(cache_dir):
+                num_files_in_dir = len(os.listdir(cache_dir))
+                if num_files_in_dir > 0:
+                    logging.info("Found {} files in cache directory {}".format(num_files_in_dir, cache_dir))
+                    logging.info(
+                        f"Deleting old cache directory for model `{model_name}` in order to prevent duplicates..."
+                    )
+                shutil.rmtree(cache_dir, ignore_errors=True)
+
+            if not os.path.exists(save_path):
+                logging.info(f"Downloading {model_name} from HuggingFace Hub to path: {save_path}")
+                os.makedirs(save_path, exist_ok=True)
+
+            path = snapshot_download(
+                repo_id=model_name,
+                library_name='nemo',
+                library_version=nemo.__version__,
+                force_download=refresh_cache,
+                cache_dir=save_path,
+                local_dir=save_path,
+                local_dir_use_symlinks=False,
+                token=hf_token,
+            )
 
         # Cannot pre-resolve the specific class without double instantiation (first for config, second for model params)
         # Default to current class, and perform basic class path resolution (handled via restore_from() + target class)
@@ -959,6 +980,29 @@ class Model(Typing, Serialization, FileIO):
 
         return class_, path
 
+    def generate_model_card(
+        self, type: str = "hf", template: str = None, template_kwargs: Optional[Dict[str, str]] = None
+    ) -> object:
+        """
+        Generates a ModelCard for the current model. This method is called when pushing the model to the Hub.
+
+        Returns:
+            An object that can be represented as a str representation of the model card, usually in Markdown format.
+        """
+        if template is None:
+            template = copy.deepcopy(NEMO_DEFAULT_MODEL_CARD_TEMPLATE)
+
+        # Populate template kwargs with common model card fields
+        if template_kwargs is None:
+            template_kwargs = {}
+
+        if type == "hf":
+            # Use HuggingFaceFileIO method to generate the huggingface model card
+            return self._get_hf_model_card(template=template, template_kwargs=template_kwargs)
+
+        else:
+            raise ValueError(f"Model card type {type} not supported.")
+
 
 class typecheck:
     """
@@ -1027,8 +1071,14 @@ class typecheck:
 
         self.ignore_collections = ignore_collections
 
+    def __call__(self, wrapped):
+        return self.wrapped_call(wrapped)
+
+    def unwrapped_call(self, wrapped):
+        return wrapped
+
     @wrapt.decorator(enabled=is_typecheck_enabled)
-    def __call__(self, wrapped, instance: Typing, args, kwargs):
+    def wrapped_call(self, wrapped, instance: Typing, args, kwargs):
         """
         Wrapper method that can be used on any function of a class that implements :class:`~nemo.core.Typing`.
         By default, it will utilize the `input_types` and `output_types` properties of the class inheriting Typing.
@@ -1114,3 +1164,34 @@ class typecheck:
             yield
         finally:
             typecheck.set_typecheck_enabled(enabled=True)
+
+    @staticmethod
+    def set_semantic_check_enabled(enabled: bool = True):
+        """
+        Global method to enable/disable semantic typechecking.
+
+        Args:
+            enabled: bool, when True will enable semantic typechecking.
+        """
+        global _TYPECHECK_SEMANTIC_CHECK_ENABLED
+        _TYPECHECK_SEMANTIC_CHECK_ENABLED = enabled
+
+    @staticmethod
+    @contextmanager
+    def disable_semantic_checks():
+        """
+        Context manager that temporarily disables semantic type checking within its context.
+        """
+        typecheck.set_semantic_check_enabled(enabled=False)
+        try:
+            yield
+        finally:
+            typecheck.set_semantic_check_enabled(enabled=True)
+
+    @staticmethod
+    def enable_wrapping(enabled: bool = True):
+        typecheck.set_typecheck_enabled(enabled)
+        if enabled:
+            typecheck.__call__ = nemo.core.classes.common.typecheck.wrapped_call
+        else:
+            typecheck.__call__ = nemo.core.classes.common.typecheck.unwrapped_call
